-- Author      : Kurapica
-- Create Date : 2012/07/25
-- Change Log  :

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.IFCooldownStatus", version) then
	return
end

----------------------------------------------------------
--- IFCooldownStatus
-- @type Interface
-- @name IFCooldownStatus
----------------------------------------------------------
interface "IFCooldownStatus"
	extend "IFCooldown"
	
	local GetTime = GetTime
	local next = next
	local pairs = pairs
	local floor = math.floor
	local ceil = math.ceil
	local random = math.random
	
	_IFCooldownStatus_List = _IFCooldownStatus_List or {}
	
	_IFCooldownStatus_Timer = Timer("IGAS_IFCooldownStatus_Timer")
	_IFCooldownStatus_Timer.Enabled = false
	_IFCooldownStatus_Timer.Interval = 0.5	-- Keep low

	function _IFCooldownStatus_Timer:OnTimer()
		local now = GetTime()
		
		for frame in pairs(_IFCooldownStatus_List) do
			if now >= frame.__IFCooldownStatus_End then
				-- Clear status
				_IFCooldownStatus_List[frame] = nil
				frame:GetChild("CooldownStatus"):Hide()
				
				if not next(_IFCooldownStatus_List) then
					_IFCooldownStatus_Timer.Enabled = false
				end
			else
				-- Update status
				if frame.IFCooldownStatusReverse then
					frame:GetChild("CooldownStatus").Value = frame.__IFCooldownStatus_Duration - (frame.__IFCooldownStatus_End - now)
				else
					frame:GetChild("CooldownStatus").Value = frame.__IFCooldownStatus_End - now
				end
			end
		end
	end

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	function Dispose(self)		
		_IFCooldownStatus_List[self] = nil
		
		if not next(_IFCooldownStatus_List) then
			_IFCooldownStatus_Timer.Enabled = false
		end
	end
	
	------------------------------------
	--- Custom the statusbar
	-- @name SetUpCooldownStatus
	-- @class function
	-- @param status
	------------------------------------
	function SetUpCooldownStatus(self, status)
		status:SetPoint("TOPLEFT", self, "TOPRIGHT")
		status:SetPoint("BOTTOMLEFT", self, "BOTTOMRIGHT")
		status.Width = 100
		
		status.StatusBarTexturePath = [[Interface\TargetingFrame\UI-StatusBar]]
		status.MinMaxValue = MinMax(1, 100)
		status.Value = 0
		status.StatusBarColor = ColorType(random(100)/100, random(100)/100, random(100)/100)
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- IFCooldownStatusReverse
	property "IFCooldownStatusReverse" {
		Get = function(self)
			return self.__IFCooldownStatusReverse
		end,
		Set = function(self, flag)
			self.__IFCooldownStatusReverse = flag
		end,
		Type = Boolean,
	}
	
	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnCooldownUpdate(self, start, duration)
		local status = self:GetChild("CooldownStatus")
		
		if start and start > 0 and duration and duration > 0 then
			-- Insert to update list
			self.__IFCooldownStatus_End = start + duration
			self.__IFCooldownStatus_Duration = duration
			
			status.MinMaxValue = MinMax(0, duration)
			status.Visible = true
			
			_IFCooldownStatus_List[self] = true
			_IFCooldownStatus_Timer.Enabled = true
		elseif _IFCooldownStatus_List[self] then
			-- Remove from update list
			_IFCooldownStatus_List[self] = nil
			
			status.Visible = false
			
			if not next(_IFCooldownStatus_List) then
				_IFCooldownStatus_Timer.Enabled = false
			end
		end
	end
	
	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function IFCooldownStatus(self)
		if not Reflector.ObjectIsClass(self, Frame) then return end
	
		self:SetUpCooldownStatus(StatusBar("CooldownStatus", self))
		
		self.OnCooldownUpdate = self.OnCooldownUpdate + OnCooldownUpdate
	end
endinterface "IFCooldownStatus"
