-- Author      : Kurapica
-- Create Date : 2012/06/28
-- Change Log  :

-- Check Version
local version = 4
if not IGAS:NewAddon("IGAS.Widget.IFMovableResizable", version) then
	return
end

----------------------------------------------------------
-- Manage Functions
----------------------------------------------------------
do
	abs = abs
	pairs = pairs
	Sleep = System.Threading.Sleep

	_Global = "Global"
	_SELECTPERCENT = 5
	_FixRange = 3
	_ISMoving = false

	_WeakMode = {__mode = "k"}

	_UsingMask = nil
	_FixPosing = false
	_PointsPool = {}
	_OwnFrames = _OwnFrames or setmetatable({}, _WeakMode)

	_ArrayMask = Array(Frame)
	_ArrayMask:ActiveThread("OnMouseDown")

	_GroupListMovable = _GroupListMovable or setmetatable({},
		{
			__index = function(self, group)
				group = tostring(group or _Global):upper()

				if not rawget(self, group) then
					rawset(self, group, setmetatable({}, _WeakMode))
				end

				return rawget(self, group)
			end,
			__call = function(self, group)
				group = tostring(group or _Global):upper()
				
				return rawget(self, group)
			end,
		}
	)

	_GroupListResizable = _GroupListResizable or setmetatable({}, getmetatable(_GroupListMovable))

	_FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 8,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}

	_FixPos = {
		TOP = Frame("IFMovableResizable_FixPos_TOP"),
		BOTTOM = Frame("IFMovableResizable_FixPos_BOTTOM"),
		LEFT = Frame("IFMovableResizable_FixPos_LEFT"),
		RIGHT = Frame("IFMovableResizable_FixPos_RIGHT"),
	}

	for dir, frm in pairs(_FixPos) do
		local txt = Texture("Texture", frm)

		frm.Visible = false
		frm.TopLevel = true
		frm.FrameStrata = "TOOLTIP"

		txt:SetAllPoints(frm)
		txt:SetTexture(1, 1, 1, 1)
	end

	_FixPos.TOP.Height = 2
	_FixPos.BOTTOM.Height = 2
	_FixPos.LEFT.Width = 2
	_FixPos.RIGHT.Width = 2

	function GetPos(frame, point, top, bottom, left, right)
		local x, y = frame:GetCenter()

		if point:find("TOP") then
			y = top or frame:GetTop()
		elseif point:find("BOTTOM") then
			y = bottom or frame:GetBottom()
		end

		if point:find("LEFT") then
			x = left or frame:GetLeft()
		elseif point:find("RIGHT") then
			x = right or frame:GetRight()
		end

		return x, y
	end

	function GetMouseArea(frame)
		local l, b, w, h = frame:GetRect()
		local e = frame:GetEffectiveScale()
		local x, y = GetCursorPosition()

		x, y = x / e, y / e
		x = x - l
		y = y - b

		local ret = ""

		if x >= 0 and x <= w and y >= 0 and y <= h then
			if y <= h / _SELECTPERCENT then
				ret = "BOTTOM"
			elseif y >= (_SELECTPERCENT - 1) * h / _SELECTPERCENT then
				ret = "TOP"
			end

			if x <= w / _SELECTPERCENT then
				ret = ret .. "LEFT"
			elseif x >= (_SELECTPERCENT - 1) * w / _SELECTPERCENT then
				ret = ret .. "RIGHT"
			end
		end

		return ret
	end

	function AutoFixPos()
		if not _UsingMask then
			return
		end

		local near, frm

		_FixPosing = true

		local mark = _ArrayMask[_UsingMask]

		while _FixPosing do
			Sleep(0.1)

			for dir, posFrm in pairs(_FixPos) do
				posFrm:ClearAllPoints()

				if dir == "TOP" then
					method = "GetTop"
				elseif dir == "BOTTOM" then
					method = "GetBottom"
				elseif dir == "LEFT" then
					method = "GetLeft"
				elseif dir == "RIGHT" then
					method = "GetRight"
				end

				near = nil
				pos = mark[method](mark)

				if not pos then return end

				for i = 1, _ArrayMask.Count do
					if i ~= _UsingMask then
						frm = _ArrayMask[i]
						if frm.Visible and frm[method](frm) and abs(frm[method](frm) - pos) <= _FixRange then
							if near then
								if dir == "TOP" or dir == "BOTTOM" then
									if frm:GetLeft() < near:GetLeft() then
										near = frm
									end
								else
									if frm:GetTop() > near:GetTop() then
										near = frm
									end
								end
							else
								near = frm
							end
						end
					end
				end

				if near then
					if dir == "TOP" then
						posFrm:SetPoint("BOTTOM", near, "TOP")

						if mark:GetLeft() > near:GetLeft() then
							posFrm:SetPoint("LEFT", near, "LEFT", -16, 0)
						else
							posFrm:SetPoint("LEFT", mark, "LEFT", -16, 0)
						end

						if mark:GetRight() < near:GetRight() then
							posFrm:SetPoint("RIGHT", near, "RIGHT", 16, 0)
						else
							posFrm:SetPoint("RIGHT", mark, "RIGHT", 16, 0)
						end
					elseif dir == "BOTTOM" then
						posFrm:SetPoint("TOP", near, "BOTTOM")

						if mark:GetLeft() > near:GetLeft() then
							posFrm:SetPoint("LEFT", near, "LEFT", -16, 0)
						else
							posFrm:SetPoint("LEFT", mark, "LEFT", -16, 0)
						end

						if mark:GetRight() < near:GetRight() then
							posFrm:SetPoint("RIGHT", near, "RIGHT", 16, 0)
						else
							posFrm:SetPoint("RIGHT", mark, "RIGHT", 16, 0)
						end
					elseif dir == "LEFT" then
						posFrm:SetPoint("RIGHT", near, "LEFT")

						if mark:GetTop() < near:GetTop() then
							posFrm:SetPoint("TOP", near, "TOP", 0, 16)
						else
							posFrm:SetPoint("TOP", mark, "TOP", 0, 16)
						end

						if mark:GetBottom() > near:GetBottom() then
							posFrm:SetPoint("BOTTOM", near, "BOTTOM", 0, -16)
						else
							posFrm:SetPoint("BOTTOM", mark, "BOTTOM", 0, -16)
						end
					elseif dir == "RIGHT" then
						posFrm:SetPoint("LEFT", near, "RIGHT")

						if mark:GetTop() < near:GetTop() then
							posFrm:SetPoint("TOP", near, "TOP", 0, 16)
						else
							posFrm:SetPoint("TOP", mark, "TOP", 0, 16)
						end

						if mark:GetBottom() > near:GetBottom() then
							posFrm:SetPoint("BOTTOM", near, "BOTTOM", 0, -16)
						else
							posFrm:SetPoint("BOTTOM", mark, "BOTTOM", 0, -16)
						end
					end

					posFrm.Near = near
					posFrm.Visible = true
				else
					posFrm.Near = nil
					posFrm.Visible = false
				end
			end
		end

		-- End Operation
		mark:StopMovingOrSizing()

		frm = mark.Owner

		_UsingMask = nil

		if frm then
			-- Get Pos
			local top, bottom, left, right

			top = mark:GetTop()
			bottom = mark:GetBottom()
			left = mark:GetLeft()
			right = mark:GetRight()

			if _FixPos["TOP"].Visible then
				top = _FixPos["TOP"].Near:GetTop()
			end

			if _FixPos["BOTTOM"].Visible then
				bottom = _FixPos["BOTTOM"].Near:GetBottom()
			end

			if _FixPos["LEFT"].Visible then
				left = _FixPos["LEFT"].Near:GetLeft()
			end

			if _FixPos["RIGHT"].Visible then
				right = _FixPos["RIGHT"].Near:GetRight()
			end

			for dir, posFrm in pairs(_FixPos) do
				posFrm.Visible = false
				posFrm.Near = nil
				posFrm:ClearAllPoints()
			end

			frm:ClearAllPoints()

			frm.Width = mark.Width
			frm.Height = mark.Height

			for point, detail in pairs(_PointsPool) do
				local x, y = GetPos(mark, point, top, bottom, left, right)
				local rx, ry = GetPos(detail.relativeTo, detail.relativePoint)

				frm:SetPoint(point, detail.relativeTo, detail.relativePoint, x - rx, y - ry)
			end

			wipe(_PointsPool)

			-- Fire Script
			frm:UnBlockScript("OnPositionChanged", "OnSizeChanged")

			mark:ClearAllPoints()
			mark:SetPoint("BOTTOMLEFT", frm)

			if _ISMoving then
				frm:Fire("OnPositionChanged")
			else
				frm:Fire("OnSizeChanged")
			end
		end
	end

	function _ArrayMask:OnMouseDown(index)
		local ret = ""

		if not _ArrayMask[index].Owner then
			return
		end
		
		if _FixPosing then
			return
		end

		local frm = _ArrayMask[index].Owner

		if _ArrayMask[index].ResizableOn then
			ret = GetMouseArea(_ArrayMask[index])
		end
		
		if not _ArrayMask[index].MovableOn and ret == "" then
			return
		end

		wipe(_PointsPool)

		for i = 1, frm:GetNumPoints() do
			local point, relativeTo, relativePoint, xOffset, yOffset = frm:GetPoint(i)
			_PointsPool[point] = {	point = point,
									relativeTo = relativeTo,
									relativePoint = relativePoint,
									xOffset = xOffset,
									yOffset = yOffset,}
		end

		frm:BlockScript("OnPositionChanged", "OnSizeChanged")

		_ArrayMask[index]:ClearAllPoints()
		_ArrayMask[index]:SetPoint("BOTTOMLEFT", frm:GetLeft(), frm:GetBottom())
		frm:ClearAllPoints()
		frm:SetAllPoints(_ArrayMask[index])

		_UsingMask = index

		if ret == "" then
			if _ArrayMask[index].MovableOn then
				_ISMoving = true
				_ArrayMask[index]:StartMoving()
			else
				return
			end
		else
			_ISMoving = false
			_ArrayMask[index]:StartSizing(ret)
		end

		return AutoFixPos()
	end

	function _ArrayMask:OnMouseUp(index)
		_FixPosing = false
	end

	function _MaskOn(IF, group)
		local lst

		if IF == IFMovable then
			lst = _GroupListMovable(group)
		elseif IF == IFResizable then
			lst = _GroupListResizable(group)
		end
		
		if not lst or not next(lst) then
			return
		end

		local cnt = 1

		for frm in pairs(lst) do
			if not frm.__IFMovableResizable_Mask then
				while _ArrayMask[cnt] and _ArrayMask[cnt].Owner do
					cnt = cnt + 1
				end

				if not _ArrayMask[cnt] then
					_ArrayMask:Insert(Frame("IFMovableResizable_Mask_"..cnt))

					_ArrayMask[cnt].TopLevel = true
					_ArrayMask[cnt].FrameStrata = "TOOLTIP"
					_ArrayMask[cnt].Movable = true
					_ArrayMask[cnt].Resizable = true
					_ArrayMask[cnt].MouseEnabled = true

					_ArrayMask[cnt].Backdrop = _FrameBackdrop
					_ArrayMask[cnt].BackdropColor = ColorType(0, 1, 0, 0.4)
				end

				frm.__IFMovableResizable_Mask = _ArrayMask[cnt]

				_ArrayMask[cnt].Owner = frm
				_ArrayMask[cnt].OwnerVisible = frm.Visible

				_ArrayMask[cnt]:ClearAllPoints()
				_ArrayMask[cnt]:SetPoint("BOTTOMLEFT", frm)

				_ArrayMask[cnt].Width = frm.Width
				_ArrayMask[cnt].Height = frm.Height

				frm.Visible = true
				_ArrayMask[cnt].Visible = true
			end

			if IF == IFMovable then
				frm.__IFMovableResizable_Mask.MovableOn = true
			elseif IF == IFResizable then
				frm.__IFMovableResizable_Mask.ResizableOn = true
			end
		end
	end

	function _MaskOff(IF, group)
		if _UsingMask then
			error("Can't turn off mode when mouse is down", 3)
		end

		local lst

		if IF == IFMovable then
			lst = _GroupListMovable(group)
		elseif IF == IFResizable then
			lst = _GroupListResizable(group)
		end
		
		if not lst or not next(lst) then
			return
		end

		local mask

		for frm in pairs(lst) do
			mask = frm.__IFMovableResizable_Mask
			if mask then
				if IF == IFMovable then
					mask.MovableOn = nil
				elseif IF == IFResizable then
					mask.ResizableOn = nil
				end

				if not mask.MovableOn and not mask.ResizableOn then
					mask.Owner.Visible = mask.OwnerVisible
					mask.Owner.__IFMovableResizable_Mask = nil
					mask.Owner = nil
					mask:ClearAllPoints()
					mask.Visible = false
				end
			end
		end
	end
end

----------------------------------------------------------
--- IFMovable
-- @type Interface
-- @name IFMovable
----------------------------------------------------------
interface "IFMovable"
	------------------------------------------------------
	-- Script
	------------------------------------------------------
	------------------------------------
	--- ScriptType, Run when the object is moved by cursor
	-- @name OnPositionChanged
	-- @type script
	-- @usage function obj:OnPositionChanged()<br>
	--    -- do someting<br>
	-- end
	------------------------------------
	script "OnPositionChanged"

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Start move registered Object
	-- @name _ModeOn
	-- @class function
	-- @param group group name
	-- @usage IFMovable._ModeOn("Unit")
	------------------------------------
	function _ModeOn(group)
		return _MaskOn(IFMovable, group)
	end

	------------------------------------
	--- Stop move registered Object
	-- @name _ModeOff
	-- @class function
	-- @param group group name
	-- @usage IFMovable._ModeOff("Unit")
	------------------------------------
	function _ModeOff(group)
		return _MaskOff(IFMovable, group)
	end

	------------------------------------
	--- Get all group name
	-- @name _GetGroupList
	-- @class function
	-- @return group name list
	-- @usage IFMovable._GetGroupList()
	------------------------------------
	function _GetGroupList()
		local ret = {}
		
		for grp in pairs(_GroupListMovable) do
			tinsert(ret, grp)
		end
		
		sort(ret)
		
		return ret
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------
	property "IFMovingGroup" {
		Get = function(self)
			return _Global
		end,
	}

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function IFMovable(self)
		if Reflector.ObjectIsClass(self, Region) then
			_GroupListMovable[self.IFMovingGroup][self] = true
		end
	end
endinterface "IFMovable"

----------------------------------------------------------
--- IFResizable
-- @type Interface
-- @name IFResizable
----------------------------------------------------------
interface "IFResizable"
	------------------------------------------------------
	-- Script
	------------------------------------------------------
	------------------------------------
	--- ScriptType, Run when a frame's size changes
	-- @name OnSizeChanged
	-- @type script
	-- @usage function obj:OnSizeChanged()<br>
	--    -- do someting<br>
	-- end
	------------------------------------
	script "OnSizeChanged"

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Start resize registered Object
	-- @name _ModeOn
	-- @class function
	-- @param group group name
	-- @usage IFResizingGroup._ModeOn("Unit")
	------------------------------------
	function _ModeOn(group)
		return _MaskOn(IFResizable, group)
	end

	------------------------------------
	--- Stop resize registered Object
	-- @name _ModeOff
	-- @class function
	-- @param group group name
	-- @usage IFResizingGroup._ModeOff("Unit")
	------------------------------------
	function _ModeOff(group)
		return _MaskOff(IFResizable, group)
	end

	------------------------------------
	--- Get all group name
	-- @name _GetGroupList
	-- @class function
	-- @return group name list
	-- @usage IFResizingGroup._GetGroupList()
	------------------------------------
	function _GetGroupList()
		local ret = {}
		
		for grp in pairs(_GroupListResizable) do
			tinsert(ret, grp)
		end
		
		sort(ret)
		
		return ret
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------
	property "IFResizingGroup" {
		Get = function(self)
			return _Global
		end,
	}

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function IFResizable(self)
		if Reflector.ObjectIsClass(self, Region) then
			_GroupListResizable[self.IFResizingGroup][self] = true
		end
	end
endinterface "IFResizable"