---------------------------------------------------------------------------------------------------------------------------------------
--- ItemList is used to display a list of items, can only be created by ItemButton
-- <br><br>inherit <a href="..\Base\VirtualUIObject.html">VirtualUIObject</a> For all methods, properties and scriptTypes
-- @name ItemList
-- @class table
-- @field Alpha Set or get the frame's transparency value(0-1)
-- @field Filter a function that used by the itemlist to check the container and inventory, if true then display it<br>Sample Filter: function(self, item, bag, slot)<br>    -- self: the itemlist's parent(itembutton)<br>    -- item: the item link<br>    -- bag: if -1 means inventory, others the bagid<br>    -- slot: the slot id in inventory of bag<br>    if bag > -1 then return true end<br>end
-- @field Direction the diplay direction, can be :TOP, BOTTOM, LEFT, RIGHT
-- @field Unit who's inventory to be checked, if Unit is "player", then the bag will also be checked.
-- @field ShowNilButton whether show nil button or not
-- @field AllowSameItem whether allow same item to be displayed
-- @field Visible the visible of the item list
-- @field Backdrop the backdrop of the item list
-- @field BackdropBorderColor the shading color for the frame's border graphic
-- @field BackdropColor the shading color for the frame's background graphic
-- @field ClampRectInsets offsets from the frame's edges used when limiting user movement or resizing of the frame
-- @field HitRectInsets the insets from the frame's edges which determine its mouse-interactable area
-- @field Depth the 3D depth of the frame (for stereoscopic 3D setups)
-- @field DepthIgnored whether the frame's depth property is ignored (for stereoscopic 3D setups)
---------------------------------------------------------------------------------------------------------------------------------------
do
	-- Check Version
	local version = 2

	if not IGAS:NewAddon("IGAS.Widget.ItemButton", version) then
		return
	end

		local ItemSize = 37
	
	-- ItemListDirection
	local _EnumDir = IGAS:NewEnum("ItemListDirection", {
		"TOP",
		"BOTTOM",
		"LEFT",
		"RIGHT",
	})

	local _PropDir = IGAS:NewPropertyType("ItemListDirection", {
		Type = "enum",
		["EnumType"] = _EnumDir,
	})

	local _Name = "ItemList"
	local _Inherit = "VirtualUIObject"
	local _ItemListContainer = Frame("IGAS_GUI_ListContainer", WorldFrame)

	_ItemListContainer.__ShowList = _ItemListContainer.__ShowList or nil
	if not _ItemListContainer.__Timer then
		local timer = Timer("Timer", _ItemListContainer)
		timer.Interval = 0
		timer.OnTimer = function(self)
			self.Interval = 0
			if self.Parent.__ShowList then
				self.Parent.__ShowList.Visible = false
			end
		end
		_ItemListContainer.__Timer = timer
	end

	-- Scripts
	local function OnShow(self)
		if _ItemListContainer.__ShowList and _ItemListContainer.__ShowList ~= self then
			_ItemListContainer.__ShowList.Visible = false
		end

		_ItemListContainer.__Timer.Interval = 2

		_ItemListContainer.__ShowList = self
	end

	local function OnHide(self)
		if _ItemListContainer.__ShowList and _ItemListContainer.__ShowList == self then
			_ItemListContainer.__ShowList = nil
		end

		_ItemListContainer.__Timer.Interval = 0
	end

	local function OnEnter(self)
		_ItemListContainer.__Timer.Interval = 0
	end

	local function OnLeave(self)
		_ItemListContainer.__Timer.Interval = 2
	end

	local function Item_OnEnter(self)
		self.Parent:Fire("OnEnter")
	end

	local function Item_OnLeave(self)
		self.Parent:Fire("OnLeave")
	end

	local function Item_OnClick(self)
		self.Parent.__Mask:Fire("OnClick", self.Item)
		self.Parent.Visible = false
	end

	local function makePosition(self, start, count)
		local btn, locX, locY
		local i, maxC = start, start + (count or 0)

		btn = self:GetChild("ItemButton"..i)

		while (not count or i < maxC) and btn do
			btn = self:GetChild("ItemButton"..i)
			if btn then
				if self.__Direction == "TOP" then
					locX = 0
					locY = ItemSize * i
				elseif self.__Direction == "BOTTOM" then
					locX = 0
					locY = - ItemSize * i
				elseif self.__Direction == "LEFT" then
					locX = - ItemSize * i
					locY = 0
				elseif self.__Direction == "RIGHT" then
					locX = ItemSize * i
					locY = 0
				end
				btn:ClearAllPoints()
				btn:SetPoint(self.__Direction, self, self.__Direction, locX, locY)
			end
			i = i + 1
			btn = self:GetChild("ItemButton"..i)
		end
	end

	local function setItem(self, index, link)
		local btn
		btn = self:GetChild("ItemButton"..index)

		if not btn then
			btn = ItemButton("ItemButton"..index, self)

			btn.ShowItemCount = false
			btn.ShowItemName = false

			btn.OnEnter = Item_OnEnter
			btn.OnLeave = Item_OnLeave
			btn.OnClick = Item_OnClick

			makePosition(self, index, 1)
		end
		btn.Item = link
		btn.Visible = true
	end

	local function makeList(self)
		local index = 1
		local item

		self.__Mask:Clear()

		if self.__FilterFunc then
			-- Set List
			--- Check Inventory Item
			for slot = 0, 19 do
				item = GetInventoryItemLink(self.__Unit, slot)
				if item and (self.__AllowSameItem or (not self.__Mask:HasItem(item))) and self.__FilterFunc(self.__Mask.Parent, item, -1, slot) then
					setItem(self, index, item)
					index = index + 1
				end
			end

			-- Check Container Item
			if self.__Unit == "player" then
				for i=0,NUM_BAG_FRAMES do
					for j=1,GetContainerNumSlots(i) do
						item = GetContainerItemLink(i,j)
						if item and (self.__AllowSameItem or (not self.__Mask:HasItem(item))) and self.__FilterFunc(self.__Mask.Parent, item, -1, slot) then
							setItem(self, index, item)
							index = index + 1
						end
					end
				end
			end

			-- A nil ItemButton
			if self.__ShowNilButton then
				setItem(self, index, nil)
				index = index + 1
			end

			-- Hide no-use ItemButton
			while self:GetChild("ItemButton"..index) do
				self:GetChild("ItemButton"..index).Item = nil
				self:GetChild("ItemButton"..index).Visible = false
				index = index + 1
			end
		else
			-- Hide List
			if self.__ShowNilButton then
				setItem(self, index, nil)
				index = index + 1
			end

			-- Hide no-use ItemButton
			while self:GetChild("ItemButton"..index) do
				self:GetChild("ItemButton"..index).Item = nil
				self:GetChild("ItemButton"..index).Visible = false
				index = index + 1
			end
		end
	end

	-- ScriptType
	local _ScriptType = {
		------------------------------------
		--- ScriptType, Run when the button is clicked
		-- @name ItemList:OnClick
		-- @class function
		-- @param button Name of the mouse button responsible for the click action:Button4, Button5, LeftButton, MiddleButton, RightButton
		-- @param down True for a mouse button down action; false for button up or other actions
		-- @usage function ItemList:OnClick(button, down)<br>
		--    -- do someting<br>
		-- end
		------------------------------------
		script "OnClick"
	}

	-- Function
	local _Function = {
		------------------------------------
		--- Refresh the item list.
		-- @name ItemList:Refresh
		-- @class function
		-- @usage ItemList:Refresh()
		------------------------------------

		-- Refresh
		function Refresh(self)
			self = self.__ItemList
			makeList(self)
		end,

		------------------------------------
		--- Add an item to the list
		-- @name ItemList:Add
		-- @class function
		-- @param item the item to be added
		-- @usage ItemList:Add([XXX' blade])
		------------------------------------

		-- Add
		function Add(self, item)
			local index = 1
			local btn

			self = self.__ItemList
			btn = self:GetChild("ItemButton"..index)
			while btn and btn.Visible and btn.Item do
				index = index + 1
				btn = self:GetChild("ItemButton"..index)
			end

			setItem(self, index, item)

			-- A nil ItemButton
			if self.__ShowNilButton then
				setItem(self, index, nil)
				index = index + 1
			end
		end,

		------------------------------------
		--- Remove an item from the list
		-- @name ItemList:Remove
		-- @class function
		-- @param item the item to be removed
		-- @usage ItemList:Remove([XXX' blade])
		------------------------------------

		-- Remove
		function Remove(self, item)
			local index = 1
			local btn, nextBtn

			self = self.__ItemList
			if not item then
				return
			end

			btn = self:GetChild("ItemButton"..index)
			while btn and btn.Visible and btn.Item ~= item do
				index = index + 1
				btn = self:GetChild("ItemButton"..index)
			end

			if btn and btn.Item == item then
				index = index + 1
				nextBtn = self:GetChild("ItemButton"..index)
				while nextBtn and nextBtn.Visible do
					btn.Item = nextBtn.Item
					btn = nextBtn
					index = index + 1
					nextBtn = self:GetChild("ItemButton"..index)
				end
				btn.Item = nil
				btn.Visible = false
			end
		end,

		------------------------------------
		--- Clear the list
		-- @name ItemList:Clear
		-- @class function
		-- @usage ItemList:Clear()
		------------------------------------

		-- Clear
		function Clear(self)
			local index = 1
			local btn

			self = self.__ItemList
			btn = self:GetChild("ItemButton"..index)
			while btn and btn.Visible do
				btn.Item = nil
				btn.Visible = false

				index = index + 1
				btn = self:GetChild("ItemButton"..index)
			end

			-- A nil ItemButton
			if self.__ShowNilButton then
				setItem(self, 1, nil)
			end
		end,

		------------------------------------
		--- Check if the item list has an item
		-- @name ItemList:HasItem
		-- @class function
		-- @param item the item to be checked
		-- @usage ItemList:HasItem([XXX' blade])
		------------------------------------

		-- HasItem
		function HasItem(self, item)
			local index = 1
			local btn, nextBtn

			self = self.__ItemList
			if not item then
				return
			end

			btn = self:GetChild("ItemButton"..index)
			while btn and btn.Visible and btn.Item ~= item do
				index = index + 1
				btn = self:GetChild("ItemButton"..index)
			end

			if btn and btn.Item == item then
				return true
			else
				return false
			end
		end,

		------------------------------------
		--- Return this object's alpha (transparency) value.
		-- @name ItemList:GetAlpha
		-- @class function
		-- @return this object's alpha (transparency) value
		-- @usage ItemList:GetAlpha()
		------------------------------------

		-- GetAlpha
		function GetAlpha(self)
			return self.__ItemList:GetAlpha() or 1
		end,

		------------------------------------
		--- Set the object's alpha (transparency) value.
		-- @name ItemList:SetAlpha
		-- @class function
		-- @param alpha this object's alpha (transparency) value
		-- @usage ItemList:SetAlpha(1)
		------------------------------------

		-- SetAlpha
		function SetAlpha(self, alpha)
			self.__ItemList:SetAlpha(alpha)
		end,

		------------------------------------
		--- Clear all attachment points for this object.
		-- @name ItemList:ClearAllPoints
		-- @class function
		-- @usage ItemList:ClearAllPoints()
		------------------------------------

		-- ClearAllPoints
		function ClearAllPoints(self)
			self.__ItemList:ClearAllPoints()
		end,

		------------------------------------
		--- Get the y location of the bottom edge of this frame
		-- @name ItemList:GetBottom
		-- @class function
		-- @return the y location of the bottom edge of this frame
		-- @usage ItemList:GetBottom()
		------------------------------------

		-- GetBottom
		function GetBottom(self)
			return self.__ItemList:GetBottom()
		end,

		------------------------------------
		--- Get the coordinates of the center of this frame
		-- @name ItemList:GetCenter
		-- @class function
		-- @return x - Distance from the left edge of the screen to the center of the region (in pixels)
		-- @return y - Distance from the bottom edge of the screen to the center of the region (in pixels)
		-- @usage ItemList:GetCenter()
		------------------------------------

		-- GetCenter
		function GetCenter(self)
			return self.__ItemList:GetCenter()
		end,

		------------------------------------
		--- Get the height of this object.
		-- @name ItemList:GetHeight
		-- @class function
		-- @return the height of this object
		-- @usage ItemList:GetHeight()
		------------------------------------

		-- GetHeight
		function GetHeight(self)
			return self.__ItemList:GetHeight()
		end,

		------------------------------------
		--- Get the x location of the left edge of this frame
		-- @name ItemList:GetLeft
		-- @class function
		-- @return the x location of the left edge of this frame
		-- @usage ItemList:GetLeft()
		------------------------------------

		-- GetLeft
		function GetLeft(self)
			return self.__ItemList:GetLeft()
		end,

		------------------------------------
		--- Get the number of anchor points for this frame
		-- @name ItemList:GetNumPoints
		-- @class function
		-- @return the number of anchor points for this frame
		-- @usage ItemList:GetNumPoints()
		------------------------------------

		-- GetNumPoints
		function GetNumPoints(self)
			return self.__ItemList:GetNumPoints()
		end,

		------------------------------------
		--- Returns information about one of the region's anchor points
		-- @name ItemList:GetPoint
		-- @class function
		-- @param index Index of an anchor point defined for the region (between 1 and ItemList:GetNumPoints())
		-- @return point - Point on this region at which it is anchored to another (string, anchorPoint)
		-- @return relativeTo - Reference to the other region to which this region is anchored (region)
		-- @return relativePoint - Point on the other region to which this region is anchored (string, anchorPoint)
		-- @return xOffset - Horizontal distance between point and relativePoint (in pixels; positive values put point  to the right of relativePoint) (number)
		-- @return yOffset - Vertical distance between point and relativePoint (in pixels; positive values put point  below relativePoint) (number)
		-- @usage ItemList:GetPoint(1)
		------------------------------------

		-- GetPoint
		function GetPoint(self, pointNum)
			return self.__ItemList:GetPoint(pointNum)
		end,

		------------------------------------
		--- Get the x location of the right edge of this frame
		-- @name ItemList:GetRight
		-- @class function
		-- @return the x location of the right edge of this frame
		-- @usage ItemList:GetRight()
		------------------------------------

		-- GetRight
		function GetRight(self)
			return self.__ItemList:GetRight()
		end,

		------------------------------------
		--- Get the y location of the top edge of this frame
		-- @name ItemList:GetTop
		-- @class function
		-- @return the y location of the top edge of this frame
		-- @usage ItemList:GetTop()
		------------------------------------

		-- GetTop
		function GetTop(self)
			return self.__ItemList:GetTop()
		end,

		------------------------------------
		--- Get the width of this object
		-- @name ItemList:GetWidth
		-- @class function
		-- @return the width of this object
		-- @usage ItemList:GetWidth()
		------------------------------------

		-- GetWidth
		function GetWidth(self)
			return self.__ItemList:GetWidth()
		end,

		------------------------------------
		--- Set this object to hidden (it and all of its children will disappear).
		-- @name ItemList:Hide
		-- @class function
		-- @usage ItemList:Hide()
		------------------------------------

		-- Hide
		function Hide(self)
			return self.__ItemList:Hide()
		end,

		------------------------------------
		--- Set this object to shown (it will appear if its parent is visible).
		-- @name ItemList:Show
		-- @class function
		-- @usage ItemList:Show()
		------------------------------------

		-- Show
		function Show(self)
			self.__ItemList:Show()
		end,

		------------------------------------
		--- Determine if this object is shown (would be visible if its parent was visible).
		-- @name ItemList:IsShown
		-- @class function
		-- @return true if the object is shown
		-- @usage ItemList:IsShown()
		------------------------------------

		-- IsShown
		function IsShown(self)
			return self.__ItemList:IsShown()
		end,

		------------------------------------
		--- Get whether the object is visible on screen (logically (IsShown() and GetParent():IsVisible()));
		-- @name ItemList:IsVisible
		-- @class function
		-- @return true if the object is visible
		-- @usage ItemList:IsVisible()
		------------------------------------

		-- IsVisible
		function IsVisible(self)
			return self.__ItemList:IsVisible()
		end,

		------------------------------------
		--- Sets all anchor points of the region to match those of another region. If no region is specified, the region's anchor points are set to those of its parent.
		-- @name ItemList:SetAllPoints
		-- @class function
		-- @param region Reference to a region or a global name of a region
		-- @usage ItemList:SetAllPoints(UIParent)
		------------------------------------

		-- SetAllPoints
		function SetAllPoints(self, frame)
			if frame and type(frame) == "string" then
				frame = _G[frame]
			end

			if not frame or type(frame) ~= "table" then
				frame = self.Parent
			end

			self.__ItemList:SetAllPoints(frame)
		end,

		------------------------------------
		--- Sets an anchor point for the region
		-- @name ItemList:SetPoint
		-- @class function
		-- @param point Point on this region at which it is to be anchored to another (string, anchorPoint)
		-- @param relativeTo Reference to the other region to which this region is to be anchored; if nil or omitted, anchors the region relative to its parent (or to the screen dimensions if the region has no parent) (region)
		-- @param relativePoint Point on the other region to which this region is to be anchored; if nil or omitted, defaults to the same value as point (string, anchorPoint)
		-- @param xOffset Horizontal distance between point and relativePoint (in pixels; positive values put point to the right of relativePoint); if nil or omitted, defaults to 0 (number)
		-- @param yOffset Vertical distance between point and relativePoint (in pixels; positive values put point below relativePoint); if nil or omitted, defaults to 0 (number)
		-- @usage ItemList:SetPoint("CENTER", UIParent, "CENTER", 50, 40)
		------------------------------------

		-- SetPoint
		function SetPoint(self, point, relativeObject, relativePoint, xOfs, yOfs)
			self.__ItemList:SetPoint(point, relativeObject, relativePoint, xOfs, yOfs)
		end,

		-- Dispose, release resource
		function Dispose(self)
			self.__ItemList:Dispose()

			-- Call super's dispose
			self:CallSuperMethod(_Inherit, "Dispose")
		end,

		------------------------------------
		--- Returns the position and dimensions of the region
		-- @name ItemList:GetRect
		-- @class function
		-- @return left - Distance from the left edge of the screen to the left edge of the region (in pixels) (number)
		-- @return bottom - Distance from the bottom edge of the screen to the bottom of the region (in pixels) (number)
		-- @return width - Width of the region (in pixels) (number)
		-- @return height - Height of the region (in pixels)
		-- @usage ItemList:GetRect()
		------------------------------------

		-- GetRect
		function GetRect(self)
			return self.__ItemList:GetRect()
		end,

		------------------------------------
		--- Returns whether the region is currently being dragged
		-- @name ItemList:IsDragging
		-- @class function
		-- @return isDragging - 1 if the region (or its parent or ancestor) is currently being dragged; otherwise nil
		-- @usage ItemList:IsDragging()
		------------------------------------

		-- IsDragging
		function IsDragging(self)
			return self.__ItemList:IsDragging()
		end,

		------------------------------------
		--- Disable rendering of "regions" (fontstrings, textures) in the specified draw layer.
		-- @name ItemList:DisableDrawLayer
		-- @class function
		-- @param layer Name of a graphics layer
		-- @usage ItemList:DisableDrawLayer("ARTWORK")
		------------------------------------

		-- DisableDrawLayer
		function DisableDrawLayer(self, ...)
			return self.__ItemList:DisableDrawLayer(...)
		end,

		------------------------------------
		--- Enable rendering of "regions" (fontstrings, textures) in the specified draw layer.
		-- @name ItemList:EnableDrawLayer
		-- @class function
		-- @param layer Name of a graphics layer
		-- @usage ItemList:EnableDrawLayer("ARTWORK")
		------------------------------------

		-- EnableDrawLayer
		function EnableDrawLayer(self, ...)
			return self.__ItemList:EnableDrawLayer(...)
		end,

		------------------------------------
		--- Creates and returns a backdrop table suitable for use in SetBackdrop
		-- @name ItemList:GetBackdrop
		-- @class function
		-- @return A table containing the backdrop settings, or nil if the frame has no backdrop
		-- @usage ItemList:GetBackdrop()
		------------------------------------

		-- GetBackdrop
		function GetBackdrop(self, ...)
			return self.__ItemList:GetBackdrop(...)
		end,

		------------------------------------
		--- Set the backdrop of the frame according to the specification provided.
		-- @name ItemList:SetBackdrop
		-- @class function
		-- @param backdropTable Optional,A table containing the backdrop settings, or nil to remove the frame's backdrop
		-- @usage ItemList:SetBackdrop{<br>
		--    bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",<br>
		--    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",<br>
		--    tile = true,<br>
		--    tileSize = 32,<br>
		--    edgeSize = 32,<br>
		--    insets = {<br>
		--         left = 11,<br>
		--         right = 12,<br>
		--         top = 12,<br>
		--         bottom = 11,<br>
		--    },<br>
		-- }<br>
		------------------------------------

		-- SetBackdrop
		function SetBackdrop(self, backdropTable)
			return self.__ItemList:SetBackdrop(backdropTable or nil)
		end,

		------------------------------------
		--- Gets the frame's backdrop border color (r, g, b, a)
		-- @name ItemList:GetBackdropBorderColor
		-- @class function
		-- @return red Red component of the color (0.0 - 1.0)
		-- @return green Green component of the color (0.0 - 1.0)
		-- @return blue Blue component of the color (0.0 - 1.0)
		-- @return alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemList:GetBackdropBorderColor()
		------------------------------------

		-- GetBackdropBorderColor
		function GetBackdropBorderColor(self, ...)
			return self.__ItemList:GetBackdropBorderColor(...)
		end,

		------------------------------------
		--- Set the frame's backdrop's border's color.
		-- @name ItemList:SetBackdropBorderColor
		-- @class function
		-- @param red Red component of the color (0.0 - 1.0)
		-- @param green Green component of the color (0.0 - 1.0)
		-- @param blue Blue component of the color (0.0 - 1.0)
		-- @param alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemList:SetBackdropBorderColor(1, 1, 0.8, 0.4)
		------------------------------------

		-- SetBackdropBorderColor
		function SetBackdropBorderColor(self, ...)
			return self.__ItemList:SetBackdropBorderColor(...)
		end,

		------------------------------------
		--- Gets the frame's backdrop color (r, g, b, a)
		-- @name ItemList:GetBackdropColor
		-- @class function
		-- @return red Red component of the color (0.0 - 1.0)
		-- @return green Green component of the color (0.0 - 1.0)
		-- @return blue Blue component of the color (0.0 - 1.0)
		-- @return alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemList:GetBackdropColor()
		------------------------------------

		-- GetBackdropColor
		function GetBackdropColor(self, ...)
			return self.__ItemList:GetBackdropColor(...)
		end,


		------------------------------------
		--- Set the frame's backdrop color.
		-- @name ItemList:SetBackdropColor
		-- @class function
		-- @param red Red component of the color (0.0 - 1.0)
		-- @param green Green component of the color (0.0 - 1.0)
		-- @param blue Blue component of the color (0.0 - 1.0)
		-- @param alpha Alpha (opacity) for the graphic (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemList:SetBackdropColor(1, 1, 0.8, 0.4)
		------------------------------------

		-- SetBackdropColor
		function SetBackdropColor(self, ...)
			return self.__ItemList:SetBackdropColor(...)
		end,

		------------------------------------
		--- Gets the modifiers to the frame's rectangle used for clamping the frame to screen.
		-- @name ItemList:GetClampRectInsets
		-- @class function
		-- @return left Offset from the left edge of the frame to the left edge of its clamping area (in pixels)
		-- @return right Offset from the right edge of the frame's clamping area to the right edge of the frame (in pixels)
		-- @return top Offset from the top edge of the frame's clamping area to the top edge of the frame (in pixels)
		-- @return bottom Offset from the bottom edge of the frame to the bottom edge of its clamping area (in pixels)
		-- @usage ItemList:GetClampRectInsets()
		------------------------------------

		-- GetClampRectInsets
		function GetClampRectInsets(self, ...)
			return self.__ItemList:GetClampRectInsets(...)
		end,

		------------------------------------
		--- Modify the frame's rectangle used to prevent dragging offscreen.
		-- @name ItemList:SetClampRectInsets
		-- @class function
		-- @param left Offset from the left edge of the frame to the left edge of its clamping area (in pixels)
		-- @param right Offset from the right edge of the frame's clamping area to the right edge of the frame (in pixels)
		-- @param top Offset from the top edge of the frame's clamping area to the top edge of the frame (in pixels)
		-- @param bottom Offset from the bottom edge of the frame to the bottom edge of its clamping area (in pixels)
		-- @usage ItemList:SetClampRectInsets(50, -50, -50, 50)
		------------------------------------

		-- SetClampRectInsets
		function SetClampRectInsets(self, ...)
			return self.__ItemList:SetClampRectInsets(...)
		end,

		------------------------------------
		--- Returns the 3D depth of the frame (for stereoscopic 3D setups)
		-- @name ItemList:GetDepth
		-- @class function
		-- @return Apparent 3D depth of this frame relative to that of its parent frame
		-- @usage ItemList:GetDepth()
		------------------------------------

		-- GetDepth
		function GetDepth(self, ...)
			return self.__ItemList:GetDepth(...)
		end,

		------------------------------------
		--- Sets the 3D depth of the frame (for stereoscopic 3D configurations)
		-- @name ItemList:SetDepth
		-- @class function
		-- @param depth Apparent 3D depth of this frame relative to that of its parent frame
		-- @usage ItemList:SetDepth(10)
		------------------------------------

		-- SetDepth
		function SetDepth(self, ...)
			return self.__ItemList:SetDepth(...)
		end,

		------------------------------------
		--- Returns the effective alpha of a frame.
		-- @name ItemList:GetEffectiveAlpha
		-- @class function
		-- @return Effective alpha (opacity) of the region (0.0 = fully transparent, 1.0 = fully opaque)
		-- @usage ItemList:GetEffectiveAlpha()
		------------------------------------

		-- GetEffectiveAlpha
		function GetEffectiveAlpha(self, ...)
			return self.__ItemList:GetEffectiveAlpha(...)
		end,

		------------------------------------
		--- Returns the overall 3D depth of the frame (for stereoscopic 3D configurations).
		-- @name ItemList:GetEffectiveDepth
		-- @class function
		-- @return Apparent 3D depth of this frame relative to the screen
		-- @usage ItemList:GetEffectiveDepth()
		------------------------------------

		-- GetEffectiveDepth
		function GetEffectiveDepth(self, ...)
			return self.__ItemList:GetEffectiveDepth(...)
		end,

		------------------------------------
		--- Get the scale factor of this object relative to the root window.
		-- @name ItemList:GetEffectiveScale
		-- @class function
		-- @return Scale factor for the frame relative to its parent
		-- @usage ItemList:GetEffectiveScale()
		------------------------------------

		-- GetEffectiveScale
		function GetEffectiveScale(self, ...)
			return self.__ItemList:GetEffectiveScale(...)
		end,

		------------------------------------
		--- Gets the frame's hit rectangle inset distances (l, r, t, b)
		-- @name ItemList:GetHitRectInsets
		-- @class function
		-- @return left Distance from the left edge of the frame to the left edge of its mouse-interactive area (in pixels)
		-- @return right Distance from the right edge of the frame to the right edge of its mouse-interactive area (in pixels)
		-- @return top Distance from the top edge of the frame to the top edge of its mouse-interactive area (in pixels)
		-- @return bottom Distance from the bottom edge of the frame to the bottom edge of its mouse-interactive area (in pixels)
		-- @usage ItemList:GetHitRectInsets()
		------------------------------------

		-- GetHitRectInsets
		function GetHitRectInsets(self, ...)
			return self.__ItemList:GetHitRectInsets(...)
		end,

		------------------------------------
		--- Set the inset distances for the frame's hit rectangle
		-- @name ItemList:SetHitRectInsets
		-- @class function
		-- @param left Distance from the left edge of the frame to the left edge of its mouse-interactive area (in pixels)
		-- @param right Distance from the right edge of the frame to the right edge of its mouse-interactive area (in pixels)
		-- @param top Distance from the top edge of the frame to the top edge of its mouse-interactive area (in pixels)
		-- @param bottom Distance from the bottom edge of the frame to the bottom edge of its mouse-interactive area (in pixels)
		-- @usage ItemList:SetHitRectInsets(10, -10, -10, 10)
		------------------------------------

		-- SetHitRectInsets
		function SetHitRectInsets(self, ...)
			return self.__ItemList:SetHitRectInsets(...)
		end,

		------------------------------------
		--- Get the ID of this frame.
		-- @name ItemList:GetID
		-- @class function
		-- @return the ID of this frame
		-- @usage ItemList:GetID()
		------------------------------------

		-- GetID
		function GetID(self, ...)
			return self.__ItemList:GetID(...)
		end,

		------------------------------------
		--- Set the ID of this frame.
		-- @name ItemList:SetID
		-- @class function
		-- @param id A numeric identifier for the frame
		-- @usage ItemList:SetID(1)
		------------------------------------

		-- SetID
		function SetID(self, ...)
			return self.__ItemList:SetID(...)
		end,

		------------------------------------
		--- Sets whether the frame's depth property is ignored (for stereoscopic 3D setups).
		-- @name ItemList:IgnoreDepth
		-- @class function
		-- @param enable True to ignore the frame's depth property; false to disable
		-- @usage ItemList:IgnoreDepth(false)
		------------------------------------

		-- IgnoreDepth
		function IgnoreDepth(self, ...)
			return self.__ItemList:IgnoreDepth(...)
		end,

		------------------------------------
		--- Returns whether the frame's depth property is ignored (for stereoscopic 3D setups)
		-- @name ItemList:IsIgnoringDepth
		-- @class function
		-- @return 1 if the frame's depth property is ignored; otherwise nil
		-- @usage ItemList:IsIgnoringDepth()
		------------------------------------

		-- IsIgnoringDepth
		function IsIgnoringDepth(self, ...)
			return self.__ItemList:IsIgnoringDepth(...)
		end,

		------------------------------------
		--- Get the list of "children" (frames and things derived from frames) of this frame.
		-- @name ItemList:GetChilds
		-- @class function
		-- @return a list of child-frames
		-- @usage ItemList:GetChilds()
		------------------------------------

		-- GetChilds
		function GetChilds(self, ...)
			return self.__ItemList:GetChilds()
		end,

		------------------------------------
		--- Get the child of the given name
		-- @name ItemList:GetChild
		-- @class function
		-- @param name the child's name or itself
		-- @return the child
		-- @usage ItemList:GetChild("BtnOkay")
		------------------------------------

		-- GetChild
		function GetChild(self, ob)
			return self.__ItemList:GetChild(ob)
		end,
	}

	-- Property
	local _Property = {
		-- Filter
		property "Filter" {
			Set = function(self, func)
				self = self.__ItemList
				if func and type(func) ~= "function" then
					error("The parameter must be a function.", 2)
				end
				self.__FilterFunc = func
			end,

			Get = function(self)
				self = self.__ItemList
				return self.__FilterFunc
			end,

			Type = "function",
		},
		-- Direction
		property "Direction" {
			Set = function(self, dir)
				self = self.__ItemList

				if not _EnumDir[dir] then
					error("The Direction is incorrect."..dir, 2)
				end
				self.__Direction = dir

				makePosition(self, 1)
			end,

			Get = function(self)
				self = self.__ItemList
				return self.__Direction
			end,

			Type = _PropDir,
		},
		-- Unit
		property "Unit" {
			Set = function(self, unit)
				self = self.__ItemList
				self.__Unit = unit
			end,

			Get = function(self)
				self = self.__ItemList
				return self.__Unit
			end,

			Type = String,
		},
		-- ShowNilButton
		property "ShowNilButton" {
			Set = function(self, flag)
				self = self.__ItemList
				self.__ShowNilButton = (flag and true) or false
			end,

			Get = function(self)
				self = self.__ItemList
				return (self.__ShowNilButton and true) or false
			end,

			Type = Boolean,
		},
		-- AllowSameItem
		property "AllowSameItem" {
			Set = function(self, flag)
				self = self.__ItemList
				self.__AllowSameItem = (flag and true) or false
			end,

			Get = function(self)
				self = self.__ItemList
				return (self.__AllowSameItem and true) or false
			end,

			Type = Boolean,
		},
		--- Visible
		property "Visible" {
			Set = function(self, visible)
				self.__ItemList.Visible = visible
			end,

			Get = function(self)
				return self.__ItemList.Visible
			end,

			Type = Boolean,
		},

		property "Backdrop" {
			Set = function(self, backdropTable)
				self:SetBackdrop(backdropTable)
			end,

			Get = function(self)
				return self:GetBackdrop()
			end,

			Type = "Backdrop",
		},

		property "BackdropBorderColor" {
			Set = function(self, colorTable)
				self:SetBackdropBorderColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,

			Get = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropBorderColor()
				return colorTable
			end,

			Type = ColorType,
		},

		property "BackdropColor" {
			Set = function(self, colorTable)
				self:SetBackdropColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,

			Get = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropColor()
				return colorTable
			end,

			Type = ColorType,
		},

		property "ClampRectInsets" {
			Set = function(self, RectInset)
				self:SetClampRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,

			Get = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetClampRectInsets()
				return RectInset
			end,

			Type = Inset,
		},

		property "HitRectInsets" {
			Set = function(self, RectInset)
				self:SetHitRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,

			Get = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetHitRectInsets()
				return RectInset
			end,

			Type = Inset,
		},

		property "Depth" {
			Set = function(self, depth)
				self:SetDepth(depth)
			end,

			Get = function(self)
				return self:GetDepth()
			end,

			Type = Number,
		},

		property "DepthIgnored" {
			Set = function(self, enabled)
				self:IgnoreDepth(enabled)
			end,

			Get = function(self)
				return (self:IsIgnoringDepth() and true) or false
			end,

			Type = Boolean,
		},
	}

	-- New
	local _New = function(name, parent)
		local mask = CreateFrame(_Inherit, name, parent)

		local frame = Frame(nil, _ItemListContainer)
		mask.__ItemList = frame
		frame.__Mask = mask

		frame.FrameStrata = "TOOLTIP"
		frame.Height = ItemSize
		frame.Width = ItemSize
		frame.Visible = false

		frame.__Direction = "BOTTOM"
		frame.__Unit = "player"
		frame.__ShowNilButton = true

		frame.OnShow = OnShow
		frame.OnHide = OnHide
		frame.OnEnter = OnEnter
		frame.OnLeave = OnLeave

		return mask
	end

	local _WidgetInfo = {
		["Name"] = _Name,
		["Inherit"] = _Inherit,
		["Function"] = _Function,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	ItemList = IGAS:NewWidget(_WidgetInfo, ItemList)
end
