﻿-- Author      : Kurapica
-- Create Date : 2012/08/03
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- AuraPanel
-- <br><br>inherit <a href="..\Common\ElementPanel.html">ElementPanel</a> For all methods, properties and scriptTypes
-- @name AuraPanel
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.AuraPanel", version) then
	return
end

class "AuraPanel"
	inherit "Frame"
	extend "IFElementPanel""IFAura"

	_FILTER_LIST = {
		CANCELABLE = true,
		HARMFUL = true,
		HELPFUL = true,
		NOT_CANCELABLE = true,
		PLAYER = true,
		RAID = true,
	}

	local strsplit = strsplit
	local select = select
	local strtrim = strtrim
	local floor = floor
	local ceil = ceil
	local UnitAura = UnitAura
	local UnitIsUnit = UnitIsUnit

	local function CheckFilter(...)
		local ret = ""

		for i = 1, select('#', ...) do
			local part = select(i, ...)

			part = part and strtrim(part)

			if _FILTER_LIST[part] then
				if #ret > 0 then
					ret = ret .. "|" .. part
				else
					ret = part
				end
			end
		end

		return ret
	end

	class "AuraIcon"
		inherit "Frame"
		extend "IFCooldownIndicator"

		local UnitAura = UnitAura

		_DebuffTypeColor = {
			["none"]	= { r = 0.80, g = 0, b = 0 },
			["Magic"]	= { r = 0.20, g = 0.60, b = 1.00 },
			["Curse"]	= { r = 0.60, g = 0.00, b = 1.00 },
			["Disease"]	= { r = 0.60, g = 0.40, b = 0 },
			["Poison"]	= { r = 0.00, g = 0.60, b = 0 },
			[""]		= { r = 0.80, g = 0, b = 0 },
		}

		------------------------------------------------------
		-- Script
		------------------------------------------------------

		------------------------------------------------------
		-- Method
		------------------------------------------------------
		------------------------------------
		--- Refresh the icon
		-- @name Refresh
		-- @class function
		------------------------------------
		function Refresh(self, unit, filter)
			local name, rank, texture, count, dtype, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff = UnitAura(unit, self.ID, filter)

			if name then
				-- Texture
				self.Icon.TexturePath = texture

				-- Count
				if count and count > 1 then
					self.Count.Visible = true
					self.Count.Text = tostring(count)
				else
					self.Count.Visible = false
				end

				-- Stealable
				self.Stealable.Visible = not UnitIsUnit("player", unit) and isStealable

				-- Debuff
				if filter and not filter:find("HELPFUL") then
					self.Overlay.VertexColor = _DebuffTypeColor[dtype] or _DebuffTypeColor.none
					self.Overlay.Visible = true
				else
					self.Overlay.Visible = false
				end

				-- Remain
				self:OnCooldownUpdate(expires - duration, duration)

				self.Visible = true
			else
				self.Visible = false
			end
		end

		------------------------------------------------------
		-- Property
		------------------------------------------------------

		------------------------------------------------------
		-- Script Handler
		------------------------------------------------------
		local function OnEnter(self)
			if self.Visible then
				IGAS.GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT')
				IGAS.GameTooltip:SetUnitAura(self.Parent.Unit, self.ID, self.Parent.Filter)
			end
		end

		local function OnLeave(self)
			IGAS.GameTooltip.Visible = false
		end

		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
		function AuraIcon(name, parent)
			local btn = Super(name, parent)

			local icon = Texture("Icon", btn, "BORDER")
			icon:SetAllPoints(btn)

			local count = FontString("Count", btn, "OVERLAY", "NumberFontNormal")
			count:SetPoint("BOTTOMRIGHT", -1, 0)

			local overlay = Texture("Overlay", btn, "OVERLAY")
			overlay:SetAllPoints(btn)
			overlay.TexturePath = [[Interface\Buttons\UI-Debuff-Overlays]]
			overlay:SetTexCoord(.296875, .5703125, 0, .515625)

			local stealable = Texture("Stealable", btn, "OVERLAY")
			stealable.TexturePath = [[Interface\TargetingFrame\UI-TargetingFrame-Stealable]]
			stealable.BlendMode = "ADD"
			stealable:SetPoint("TOPLEFT", -3, 3)
			stealable:SetPoint("BOTTOMRIGHT", 3, -3)

			btn.OnEnter = btn.OnEnter + OnEnter
			btn.OnLeave = btn.OnLeave + OnLeave

			return btn
		end
	endclass "AuraIcon"

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Refresh totems
	-- @name Refresh
	-- @class function
	------------------------------------
	function Refresh(self)
		self:Each("Refresh", self.Unit, self.Filter)
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- Filter
	property "Filter" {
		Get = function(self)
			return self.__AuraPanelFilter
		end,
		Set = function(self, filter)
			if filter then
				filter = CheckFilter(strsplit("|", filter:upper()))

				if #filter == 0 then
					filter = nil
				end
			end

			self.__AuraPanelFilter = filter
		end,
		Type = String + nil,
	}

	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function AuraPanel(name, parent)
		local panel = Super(name, parent)

		panel:ConvertClass(AuraPanel)

		panel.FrameStrata = "LOW"
		panel.AutoSize = true
		panel.ColumnCount = 7
		panel.RowCount = 6
		panel.ElementWidth = 16
		panel.ElementHeight = 16
		panel.ElementType = AuraIcon
		panel.Count = 42

		return panel
	end
endclass "AuraPanel"