-- Author      : Kurapica
-- Create Date : 2012/06/25
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- EclipseBar
-- <br><br>inherit <a href="..\Base\StatusBar.html">StatusBar</a> For all methods, properties and scriptTypes
-- @name EclipseBar
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.EclipseBar", version) then
	return
end

class "EclipseBar"
	inherit "Frame"
	extend "IFEclipse"

	local UnitPower = UnitPower
	local abs = abs
	local unpack = unpack
	
	SPELL_POWER_ECLIPSE = SPELL_POWER_ECLIPSE
	
	ECLIPSE_MARKER_COORDS = ECLIPSE_MARKER_COORDS
	ECLIPSE_ICONS = ECLIPSE_ICONS

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	function Refresh(self)
		local power = UnitPower( self.Unit, SPELL_POWER_ECLIPSE )
		local maxPower = UnitPowerMax( self.Unit, SPELL_POWER_ECLIPSE )
		
		if maxPower == 0 then return end
		
		self.PowerText.Text = abs(power)
		
		self.Marker:SetPoint("CENTER", (power/maxPower) *  19 / 35 * self.Width, 0)
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- SunActived
	property "SunActived" {
		Get = function(self)
			return self.__SunActived or false
		end,
		Set = function(self, flag)
			if flag ~= self.SunActived then
				self.__SunActived = flag

				if flag then
					self.Glow:ClearAllPoints()
					
					local glowInfo = ECLIPSE_ICONS["sun"].big;
					self.Glow:SetPoint("CENTER", self.Sun, "CENTER", 0, 0)
					self.Glow:SetWidth(glowInfo.x)
					self.Glow:SetHeight(glowInfo.y)
					self.Glow:SetTexCoord(glowInfo.left, glowInfo.right, glowInfo.top, glowInfo.bottom)
				end

				self.SunBar.SunActivate.Playing = flag
				self.Glow.SunActivate.Playing = flag
				self.DarkMoon.SunActivate.Playing = flag
				
				self.SunBar.SunDeactivate.Playing = not flag
				self.Glow.SunDeactivate.Playing = not flag
				self.DarkMoon.SunDeactivate.Playing = not flag
			end
		end,
		Type = Boolean,
	}
	-- MoonActived
	property "MoonActived" {
		Get = function(self)
			return self.__MoonActived or false
		end,
		Set = function(self, flag)
			if flag ~= self.MoonActived then
				self.__MoonActived = flag
				
				if flag then
					self.Glow:ClearAllPoints()
					
					local glowInfo = ECLIPSE_ICONS["moon"].big;
					self.Glow:SetPoint("CENTER", self.Moon, "CENTER", 0, 0)
					self.Glow:SetWidth(glowInfo.x)
					self.Glow:SetHeight(glowInfo.y)
					self.Glow:SetTexCoord(glowInfo.left, glowInfo.right, glowInfo.top, glowInfo.bottom)
				end

				self.MoonBar.MoonActivate.Playing = flag
				self.Glow.MoonActivate.Playing = flag
				self.DarkSun.MoonActivate.Playing = flag
				
				self.MoonBar.MoonDeactivate.Playing = not flag
				self.Glow.MoonDeactivate.Playing = not flag
				self.DarkSun.MoonDeactivate.Playing = not flag
			end
		end,
		Type = Boolean,
	}
	-- EclipseDirection
	property "Direction" {
		Get = function(self)
			return self.__EclipseBar_Direction or IFEclipse.EclipseDirection.None 
		end,
		Set = function(self, dir)
			if self.__EclipseBar_Direction ~= dir then
				self.__EclipseBar_Direction = dir
				self.Marker:SetTexCoord(unpack(ECLIPSE_MARKER_COORDS[dir]))
			end
		end,
		Type = IFEclipse.EclipseDirection,
	}
	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnEnter(self)
		self.PowerText.Visible = true
	end
	
	local function OnLeave(self)
		if not self.LockShow then
			self.PowerText.Visible = false
		end
	end
	
	local function SunActivate_OnPlay(self)
		local panel = self.Parent.Parent
		
		panel.MoonBar.Alpha = 0
		panel.DarkSun.Alpha = 0
	end
	
	local function SunActivate_OnFinished(self)
		local panel = self.Parent.Parent
		
		panel.SunBar.Alpha = 1
		panel.Glow.Alpha = 1
		panel.DarkMoon.Alpha = 1
		
		panel.Glow.Pulse:Play()
	end
	
	local function SunDeactivate_OnPlay(self)
		local panel = self.Parent.Parent
		
		panel.Glow.Pulse:Stop()
	end
	
	local function SunDeactivate_OnFinished(self)
		local panel = self.Parent.Parent
		
		panel.SunBar.Alpha = 0
		panel.Glow.Alpha = 0
		panel.DarkMoon.Alpha = 0
	end
	
	local function MoonActivate_OnPlay(self)
		local panel = self.Parent.Parent
		
		panel.SunBar.Alpha = 0
		panel.DarkMoon.Alpha = 0
	end
	
	local function MoonActivate_OnFinished(self)
		local panel = self.Parent.Parent
		
		panel.MoonBar.Alpha = 1
		panel.Glow.Alpha = 1
		panel.DarkSun.Alpha = 1
		
		panel.Glow.Pulse:Play()
	end
	
	local function MoonDeactivate_OnPlay(self)
		local panel = self.Parent.Parent
		
		panel.Glow.Pulse:Stop()
	end
	
	local function MoonDeactivate_OnFinished(self)
		local panel = self.Parent.Parent
		
		panel.MoonBar.Alpha = 0
		panel.Glow.Alpha = 0
		panel.DarkSun.Alpha = 0
	end
	
	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function EclipseBar(name, parent)
		local panel = Super(name, parent)

		panel.Height = 38
		panel.Width = 140
		
		-- Bar
		local bar = Texture("Bar", panel, "ARTWORK")
		bar.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		bar:SetAllPoints(panel)
		bar:SetTexCoord(0.00390625, 0.55078125, 0.63281250, 0.92968750)
		
		-- Sun
		local sun = Texture("Sun", panel, "BACKGROUND")
		sun.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		sun.Width = 23
		sun.Height = 23
		sun:SetPoint("CENTER", bar, "RIGHT", -17, 1)
		sun:SetTexCoord(0.65625000, 0.74609375, 0.37500000, 0.55468750)
		
		-- Moon
		local moon = Texture("Moon", panel, "BACKGROUND")
		moon.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		moon.Width = 23
		moon.Height = 23
		moon:SetPoint("CENTER", bar, "LEFT", 17, 1)
		moon:SetTexCoord(0.55859375, 0.64843750, 0.57031250, 0.75000000)
		
		-- DarkSun
		local darkSun = Texture("DarkSun", panel, "BACKGROUND", nil, 1)
		darkSun.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		darkSun.Width = 23
		darkSun.Height = 23
		darkSun:SetPoint("CENTER", sun, "CENTER")
		darkSun:SetTexCoord(0.55859375, 0.64843750, 0.76562500, 0.94531250)
		
		-- DarkMoon
		local darkMoon = Texture("DarkMoon", panel, "BACKGROUND", nil, 1)
		darkMoon.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		darkMoon.Width = 23
		darkMoon.Height = 23
		darkMoon:SetPoint("CENTER", moon, "CENTER")
		darkMoon:SetTexCoord(0.55859375, 0.64843750, 0.37500000, 0.55468750)
		
		-- SunBar
		local sunBar = Texture("SunBar", panel, "ARTWORK", nil, 1)
		sunBar.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		sunBar:SetAllPoints(panel)
		sunBar:SetTexCoord(0.00390625, 0.55078125, 0.32031250, 0.61718750)
	
		-- MoonBar
		local moonBar = Texture("MoonBar", panel, "ARTWORK", nil, 1)
		moonBar.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		moonBar:SetAllPoints(panel)
		moonBar:SetTexCoord(0.00390625, 0.55078125, 0.00781250, 0.30468750)
		
		-- Marker
		local marker = Texture("Marker", panel, "OVERLAY")
		marker.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		marker.BlendMode = "ADD"
		marker.Width = 20
		marker.Height = 20
		marker:SetPoint("CENTER", 0, 2)
		marker:SetTexCoord(1.0, 0.914, 0.82, 1.0)
		
		-- PowerText
		local powerText = FontString("PowerText", panel, "OVERLAY", "TextStatusBarText")
		powerText:SetPoint("CENTER")
		powerText.Visible = false
		
		-- Glow
		local glow = Texture("Glow", panel, "OVERLAY")
		glow.TexturePath = [[Interface\PlayerFrame\UI-DruidEclipse]]
		glow.BlendMode = "ADD"
		glow.Width = 43
		glow.Height = 45
		glow:SetPoint("CENTER")
		glow:SetTexCoord(0.55859375, 0.72656250, 0.00781250, 0.35937500)
		
		-- Animation for Glow
		local glowPulse = AnimationGroup("Pulse", glow)
		glowPulse.Looping = "REPEAT"		
		
		local scalePulse1 = Scale("ScalePulse1", glowPulse)
		scalePulse1.Order = 1
		scalePulse1.Duration = 0.5
		scalePulse1.Smoothing = "IN_OUT"
		scalePulse1.Scale = Dimension(1.08, 1.08)
		
		local scalePulse2 = Scale("ScalePulse2", glowPulse)
		scalePulse2.Order = 2
		scalePulse2.Duration = 0.5
		scalePulse2.Smoothing = "IN_OUT"
		scalePulse2.Scale = Dimension(0.9259, 0.9259)
		
		-- Animation for sunActivate
		local sunActivate = AnimationGroup("SunActivate", sunBar)
		sunActivate.OnPlay = SunActivate_OnPlay
		sunActivate.OnFinished = SunActivate_OnFinished
		
		local alpha = Alpha("Alpha", sunActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1
		
		sunActivate = AnimationGroup("SunActivate", glow)
		
		alpha = Alpha("Alpha", sunActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1
		
		sunActivate = AnimationGroup("SunActivate", darkMoon)
		
		alpha = Alpha("Alpha", sunActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1		
		
		-- Animation for sunDeactivate
		local sunDeactivate = AnimationGroup("SunDeactivate", sunBar)
		sunDeactivate.OnPlay = SunDeactivate_OnPlay
		sunDeactivate.OnFinished = SunDeactivate_OnFinished
		
		alpha = Alpha("Alpha", sunDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		sunDeactivate = AnimationGroup("SunDeactivate", glow)
		
		alpha = Alpha("Alpha", sunDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		sunDeactivate = AnimationGroup("SunDeactivate", darkMoon)
		
		alpha = Alpha("Alpha", sunDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		-- Animation for moonActivate
		local moonActivate = AnimationGroup("MoonActivate", moonBar)
		moonActivate.OnPlay = MoonActivate_OnPlay
		moonActivate.OnFinished = MoonActivate_OnFinished
		
		alpha = Alpha("Alpha", moonActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1
		
		moonActivate = AnimationGroup("MoonActivate", glow)
		
		alpha = Alpha("Alpha", moonActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1
		
		moonActivate = AnimationGroup("MoonActivate", darkSun)
		
		alpha = Alpha("Alpha", moonActivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = 1		
		
		-- Animation for moonDeactivate
		local moonDeactivate = AnimationGroup("MoonDeactivate", moonBar)
		moonDeactivate.OnPlay = MoonDeactivate_OnPlay
		moonDeactivate.OnFinished = MoonDeactivate_OnFinished
		
		alpha = Alpha("Alpha", moonDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		moonDeactivate = AnimationGroup("MoonDeactivate", glow)
		
		alpha = Alpha("Alpha", moonDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		moonDeactivate = AnimationGroup("MoonDeactivate", darkSun)
		
		alpha = Alpha("Alpha", moonDeactivate)
		alpha.Order = 1
		alpha.Duration = 0.6
		alpha.Change = -1
		
		-- Script Handler
		panel.OnEnter = panel.OnEnter + OnEnter
		panel.OnLeave = panel.OnLeave + OnLeave
		
		return panel
	end
endclass "EclipseBar"