﻿-- Author      : Kurapica
-- Create Date : 2012/07/22
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- HolyPowerPanel
-- <br><br>inherit <a href="..\Base\Frame.html">Frame</a> For all methods, properties and scriptTypes
-- @name HolyPowerPanel
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.HolyPowerPanel", version) then
	return
end

class "HolyPowerPanel"
	inherit "Frame"
	extend "IFHolyPower"	
	
	MAX_HOLY_POWER = MAX_HOLY_POWER
	SPELL_POWER_HOLY_POWER = SPELL_POWER_HOLY_POWER	

	local UnitPower = UnitPower
	
	class "HolyPower"
		inherit "Frame"
		
		SPELL_POWER_HOLY_POWER = SPELL_POWER_HOLY_POWER	

		local UnitPower = UnitPower
		
		------------------------------------------------------
		-- Script
		------------------------------------------------------

		------------------------------------------------------
		-- Method
		------------------------------------------------------		
		------------------------------------
		--- Refresh the shard
		-- @name Refresh
		-- @class function
		------------------------------------
		function Refresh(self, unit)
			self.Actived = UnitPower(unit, SPELL_POWER_HOLY_POWER) >= self.ID
		end
		
		------------------------------------------------------
		-- Property
		------------------------------------------------------
		property "Actived" {
			Get = function(self)
				return self.__Actived or false
			end,
			Set = function(self, flag)
				if self.__Actived ~= flag then
					self.__Actived = flag
					
					if flag then
						self.Deactivate.Playing = false
						self.Activate.Playing = true
					else
						self.Activate.Playing = false
						self.Deactivate.Playing = true
					end
				end
			end,
			Type = Boolean,
		}
		
		------------------------------------------------------
		-- Script Handler
		------------------------------------------------------
		local function Activate_OnFinished(self)
			self.Parent.Alpha = 1
		end
		
		local function Deactivate_OnFinished(self)
			self.Parent.Alpha = 0
		end
		
		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
		function HolyPower(name, parent)			
			local rune = Super(name, parent)	
			
			local icon = Texture("Texture", rune, "OVERLAY")
			icon.TexturePath = [[Interface\PlayerFrame\PaladinPowerTextures]]
			
			-- Animation
			local activate = AnimationGroup("Activate", rune)
			activate.OnFinished = Activate_OnFinished
			
			local alphaIn = Alpha("AlphaIn", activate)
			alphaIn.Order = 1
			alphaIn.Duration = 0.2
			alphaIn.Change = 1
		
			local deactivate = AnimationGroup("Deactivate", rune)
			deactivate.OnFinished = Deactivate_OnFinished
			
			local alphaOut = Alpha("AlphaOut", deactivate)
			alphaOut.Order = 1
			alphaOut.Duration = 0.3
			alphaOut.Change = -1
			
			-- Property
			rune.__Actived = false
			
			return rune
		end
	endclass "HolyPower"

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Refresh totems
	-- @name Refresh
	-- @class function
	------------------------------------
	function Refresh(self)
		for i = 1, MAX_HOLY_POWER do
			self["HolyPower"..i]:Refresh(self.Unit)
		end
		
		if UnitPower(self.Unit, SPELL_POWER_HOLY_POWER) == MAX_HOLY_POWER then
			self.GlowBG.Looping = true
			self.GlowBG.Pulse.Playing = true
		else
			self.GlowBG.Looping = false
		end
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------

	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnFinished(self)
		if self.Parent.Looping then
			return self:Play()
		end
	end
	
	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function HolyPowerPanel(name, parent)
		local panel = Super(name, parent)
		
		panel.FrameStrata = "LOW"
		panel.Toplevel = true
		panel.Width = 136
		panel.Height = 39
		
		local back = Texture("BG", panel, "BACKGROUND", nil, -5)
		back.TexturePath = [[Interface\PlayerFrame\PaladinPowerTextures]]
		back:SetAllPoints(panel)
		back:SetTexCoord(0.00390625, 0.53515625, 0.32812500, 0.63281250)
		
		local glow = Frame("GlowBG", panel)
		glow:SetAllPoints(panel)
		glow.Alpha = 0
		
		local glowTexture = Texture("Texture", glow, "BACKGROUND", nil, -1)
		glowTexture.TexturePath = [[Interface\PlayerFrame\PaladinPowerTextures]]
		glowTexture:SetAllPoints(glow)
		glowTexture:SetTexCoord(0.00390625, 0.53515625, 0.00781250, 0.31250000)
		
		-- Animation for glow
		local pulse = AnimationGroup("Pulse", glow)
		pulse.OnFinished = OnFinished
		
		local alphaIn = Alpha("AlphaIn", pulse)
		alphaIn.Order = 1
		alphaIn.Duration = 0.5
		alphaIn.Change = 1
		
		local alphaOut = Alpha("AlphaOut", pulse)
		alphaOut.Order = 2
		alphaOut.StartDelay = 0.3
		alphaOut.Duration = 0.6
		alphaOut.Change = -1		
		
		-- Runes
		local btnHolyPower
		
		for i = 1, MAX_HOLY_POWER do
			btnHolyPower = HolyPower("HolyPower"..i, panel)
			btnHolyPower.ID = i
			btnHolyPower.Alpha = 0
			
			if i == 1 then
				btnHolyPower.Width = 36
				btnHolyPower.Height = 22
				btnHolyPower:SetPoint("TOPLEFT", 21, -11)
				btnHolyPower.Texture:SetTexCoord(0.00390625, 0.14453125, 0.64843750, 0.82031250)
			elseif i == 2 then
				btnHolyPower.Width = 31
				btnHolyPower.Height = 17
				btnHolyPower:SetPoint("LEFT", self["HolyPower1"], "RIGHT")
				btnHolyPower.Texture:SetTexCoord(0.00390625, 0.12500000, 0.83593750, 0.96875000)
			elseif i == 3 then
				btnHolyPower.Width = 27
				btnHolyPower.Height = 21
				btnHolyPower:SetPoint("LEFT", self["HolyPower2"], "RIGHT", 2, -1)
				btnHolyPower.Texture:SetTexCoord(0.15234375, 0.25781250, 0.64843750, 0.81250000)
			end
		end
		
		Refresh(panel)
		
		return panel
	end
endclass "HolyPowerPanel"