﻿-- Author      : Kurapica
-- Create Date : 2012/06/25
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- ReadyCheckIcon
-- <br><br>inherit <a href="..\Base\Texture.html">Texture</a> For all methods, properties and scriptTypes
-- @name ReadyCheckIcon
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.ReadyCheckIcon", version) then
	return
end

class "ReadyCheckIcon"
	inherit "Texture"
	extend "IFReadyCheck"
	
	import "System.Threading"

	FINISH_TIME = 10
	FADE_TIME = 1.5

	READY_CHECK_WAITING_TEXTURE = READY_CHECK_WAITING_TEXTURE
	READY_CHECK_READY_TEXTURE = READY_CHECK_READY_TEXTURE
	READY_CHECK_NOT_READY_TEXTURE = READY_CHECK_NOT_READY_TEXTURE
	READY_CHECK_AFK_TEXTURE = READY_CHECK_AFK_TEXTURE
	
	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Start the ready check
	-- @name Start
	-- @type function
	------------------------------------
	function Start(self)
		self.TexturePath = READY_CHECK_WAITING_TEXTURE
		self.Alpha = 1
		self.Visible = true
	end
	
	------------------------------------
	--- Confirm the ready check
	-- @name Confirm
	-- @type function
	-- @param ready boolean
	------------------------------------
	function Confirm(self, ready)
		if ready then
			self.TexturePath = READY_CHECK_READY_TEXTURE
		else
			self.TexturePath = READY_CHECK_NOT_READY_TEXTURE
		end
		self.Alpha = 1
		self.Visible = true
	end
	
	------------------------------------
	--- Finish the ready check
	-- @name Finish
	-- @type function
	------------------------------------
	function Finish(self)
		if self.TexturePath == READY_CHECK_WAITING_TEXTURE then
			self.TexturePath = READY_CHECK_AFK_TEXTURE
		end
		
		-- Wait to Fade
		Threading.Sleep(self.FinishTime)
		
		-- Fading
		local alpha = self.Alpha
		local perSec = alpha / self.FadeTime / 10
		
		while alpha > 0 do
			self.Alpha = alpha
			
			Threading.Sleep(0.1)
			
			alpha = alpha - perSec
		end
		
		self.Alpha = 0
		self.Visible = false
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- FinishTime
	property "FinishTime" {
		Get = function(self)
			return self.__FinishTime or FINISH_TIME
		end,
		Set = function(self, time)
			if time < 0 then
				self.__FinishTime = nil
			elseif time < 0.1 then
				self.__FinishTime = 0.1
			else
				self.__FinishTime = time
			end
		end,
		Type = Number,
	}
	-- FadeTime
	property "FadeTime" {
		Get = function(self)
			return self.__FadeTime or FADE_TIME
		end,
		Set = function(self, time)
			if time < 0 then
				self.__FadeTime = nil
			elseif time < 0.1 then
				self.__FadeTime = 0.1
			else
				self.__FadeTime = time
			end
		end,
		Type = Number,
	}

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function ReadyCheckIcon(name, parent)
		local icon = Super(name, parent)		
		
		icon.Visible = false
		
		return icon
	end
endclass "ReadyCheckIcon"