﻿-- Author      : Kurapica
-- Create Date : 2012/07/18
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- RunePanel
-- <br><br>inherit <a href="..\Common\LayoutPanel.html">LayoutPanel</a> For all methods, properties and scriptTypes
-- @name RunePanel
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.RunePanel", version) then
	return
end

class "RunePanel"
	inherit "LayoutPanel"
	extend "IFRune"
	
	local floor = math.floor
	
	MAX_RUNES = 6
	
	RUNETYPE_COMMON = 0
	RUNETYPE_BLOOD = 1
	RUNETYPE_UNHOLY = 2
	RUNETYPE_FROST = 3
	RUNETYPE_DEATH = 4
	
	IconTextures = IconTextures or {
		[RUNETYPE_BLOOD] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Blood",
		[RUNETYPE_UNHOLY] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Unholy",
		[RUNETYPE_FROST] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Frost",
		[RUNETYPE_DEATH] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Death",
	}
	
	RuneTextures = RuneTextures or {
		[RUNETYPE_BLOOD] = "Interface\\PlayerFrame\\UI-PlayerFrame-DeathKnight-Blood-Off.tga",
		[RUNETYPE_UNHOLY] = "Interface\\PlayerFrame\\UI-PlayerFrame-DeathKnight-Death-Off.tga",
		[RUNETYPE_FROST] = "Interface\\PlayerFrame\\UI-PlayerFrame-DeathKnight-Frost-Off.tga",
		[RUNETYPE_DEATH] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Chromatic-Off.tga",
	}
	
	RuneEnergizeTextures = RuneEnergizeTextures or {
		[RUNETYPE_BLOOD] = "Interface\\PlayerFrame\\Deathknight-Energize-Blood",
		[RUNETYPE_UNHOLY] = "Interface\\PlayerFrame\\Deathknight-Energize-Unholy",
		[RUNETYPE_FROST] = "Interface\\PlayerFrame\\Deathknight-Energize-Frost",
		[RUNETYPE_DEATH] = "Interface\\PlayerFrame\\Deathknight-Energize-White",
	}
	
	RuneColors = RuneColors or {
		[RUNETYPE_COMMON] = ColorType(1, 1, 1),
		[RUNETYPE_BLOOD] = ColorType(1, 0, 0),
		[RUNETYPE_UNHOLY] = ColorType(0, 0.5, 0),
		[RUNETYPE_FROST] = ColorType(0, 1, 1),
		[RUNETYPE_DEATH] = ColorType(0.8, 0.1, 1),
	}
	
	RuneMapping = RuneMapping or {
		[1] = "BLOOD",
		[2] = "UNHOLY",
		[3] = "FROST",
		[4] = "DEATH",
	}
	
	RuneBtnMapping = RuneBtnMapping or {
		[1] = 1,
		[2] = 2,
		[3] = 5,
		[4] = 6,
		[5] = 3,
		[6] = 4,
	}
	
	class "RuneButton"
		inherit "Button"
		
		local GetRuneType = GetRuneType
		local GetRuneCooldown = GetRuneCooldown
		
		ALPHA_PER = 0.2
		
		------------------------------------------------------
		-- Script
		------------------------------------------------------

		------------------------------------------------------
		-- Method
		------------------------------------------------------
		------------------------------------
		--- Starts animating the group
		-- @name Play
		-- @class function
		------------------------------------
		-- Play
		function Play(self)
			self.Glow.RuneWhiteGlow.Energize:Play()
			self.Glow.RuneColorGlow.Energize:Play()
		end
		
		------------------------------------
		--- Stops animation of the group
		-- @name Stop
		-- @class function
		------------------------------------
		-- Stop
		function Stop(self)
			self.Glow.RuneWhiteGlow.Energize:Stop()
			self.Glow.RuneColorGlow.Energize:Stop()
		end
		
		------------------------------------
		--- Refresh the rune
		-- @name Refresh
		-- @class function
		------------------------------------
		function Refresh(self, noShine)
			if self.ID then
				self.Visible = true
				
				local runeType = GetRuneType(self.ID)
				
				if self.Rune.RuneType == runeType then return end
				
				self.Rune.RuneType = runeType
				
				if runeType then
					self.Glow.RuneColorGlow.TexturePath = RuneEnergizeTextures[runeType]
					self.Shine.Texture.VertexColor = RuneColors[runeType]
					self.Rune.TexturePath = IconTextures[runeType]
					
					self.Rune.Visible = true
				
					if not noShine then
						self:Shine()
					end
				else
					self.Rune.Visible = false
				end
			else
				self.Visible = false
			end
		end
		
		------------------------------------
		--- Update cooldown by index
		-- @name UpdatePower
		-- @class function
		------------------------------------
		function UpdatePower(self, isEnergize)
			if not self.ID then return end
			
			local start, duration, runeReady = GetRuneCooldown(self.ID)			
			
			if not runeReady then
				if start then
					self.Cooldown:SetCooldown(start, duration)
					self.Cooldown.Visible = true
				end
				self:Stop();
			else
				self.Cooldown:Hide();
				self.Shine.Texture.VertexColor = RuneColors[RUNETYPE_COMMON]
				self:Shine()
			end
			
			if isEnergize then
				self:Play()
			end
		end
		
		------------------------------------
		--- Shine
		-- @name Shine
		-- @class function
		------------------------------------
		function Shine(self)
			if not self.Shine.Texture.ShineTimer.Enabled then
				self.Shine.Texture.ShineTimer._FadeIn = true
				self.Shine.Texture.ShineTimer.Enabled = true
			end
		end
		
		------------------------------------------------------
		-- Property
		------------------------------------------------------

		------------------------------------------------------
		-- Script Handler
		------------------------------------------------------
		local function OnSizeChanged(self)
			self.Glow.glowWhite.Height = self.Height / 2 - 1
			self.Glow.glowWhite.Width = self.Width / 2 - 1
		end
		
		local function OnTimer(self)
			if self._FadeIn then
				self.Parent.Alpha = self.Parent.Alpha + ALPHA_PER
				
				if self.Parent.Alpha >= 1 then
					self._FadeIn = false
				end
			else				
				self.Parent.Alpha = self.Parent.Alpha - ALPHA_PER
				
				if self.Parent.Alpha <= 0 then
					self._FadeIn = nil
					self.Enabled = false
					self.Parent.Alpha = 0
				end
			end
		end
		
		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
		function RuneButton(name, parent)
			local btn = Super(name, parent)
			
			btn.Height = 18
			btn.Width = 18
			
			-- Cooldown
			local cd = Cooldown("Cooldown", btn)
			cd.DrawEdge = true
			cd.FrameStrata = "LOW"
			cd:SetPoint("TOPLEFT", 2, -2)
			cd:SetPoint("BOTTOMRIGHT", -1, 1)
			
			-- Border
			local border = Frame("Border", btn)
			border.FrameStrata = "LOW"
			border:SetPoint("TOPLEFT", -3, 3)
			border:SetPoint("BOTTOMRIGHT", 3, -3)
			
			local borderTexture = Texture("Texture", border, "OVERLAY")
			borderTexture.TexturePath = [[Interface\PlayerFrame\UI-PlayerFrame-Deathknight-Ring]]
			borderTexture.VertexColor = ColorType(0.6, 0.6, 0.6, 1)
			borderTexture:SetAllPoints(border)
			
			border.FrameLevel = border.FrameLevel + 1
			
			-- Shine
			local shine = Frame("Shine", btn)
			shine.FrameStrata = "MEDIUM"
			shine:SetAllPoints(btn)
			
			local shineTexture = Texture("Texture", border, "OVERLAY")
			shineTexture.TexturePath = [[Interface\ComboFrame\ComboPoint]]
			shineTexture.BlendMode = "ADD"
			shineTexture.Alpha = 0
			shineTexture:SetPoint("TOPLEFT", -21, 9)
			shineTexture:SetPoint("BOTTOMRIGHT", 21, -9)
			shineTexture:SetTexCoord(0.5625, 1, 0, 1)
			
			-- Glow
			local glow = Frame("Glow", btn)
			glow.FrameStrata = "HIGH"
			glow:SetAllPoints(btn)
			
			local glowWhite = Texture("RuneWhiteGlow", glow, "OVERLAY", nil, -1)
			glowWhite.TexturePath = [[Interface\PlayerFrame\Deathknight-Energize-White]]
			glowWhite.Alpha = 0
			glowWhite.Height = 8
			glowWhite.Width = 8
			glowWhite:SetPoint("CENTER", 0, -1)
			
			local glowColor = Texture("RuneColorGlow", glow, "OVERLAY")
			glowColor.TexturePath = [[Interface\PlayerFrame\Deathknight-Energize-Blood]]
			glowColor.Alpha = 0
			glowColor:SetPoint("TOPLEFT", -7, 7)
			glowColor:SetPoint("BOTTOMRIGHT", 7, -7)
			
			-- RuneTexture
			local rune = Texture("Rune", btn, "ARTWORK")
			rune.TexturePath = [[Interface\PlayerFrame\UI-PlayerFrame-Deathknight-Blood]]
			rune:SetPoint("TOPLEFT", -3, 3)
			rune:SetPoint("BOTTOMRIGHT", 3, -3)
			
			-- Animation for RuneWhiteGlow
			local agWhite = AnimationGroup("Energize", glowWhite)
			
			local scaleWhite = Scale("ScaleWhite", agWhite)
			scaleWhite.Order = 1
			scaleWhite.Duration = 0.15
			scaleWhite.EndDelay = 1
			scaleWhite.Scale = Dimension(4, 4)
			
			local alphaWhiteOn = Alpha("AlphaWhiteOn", agWhite)
			alphaWhiteOn.Order = 1
			alphaWhiteOn.Duration = 0.2
			alphaWhiteOn.EndDelay = 1
			alphaWhiteOn.Change = 1
			
			local alphaWhiteOff = Alpha("AlphaWhiteOff", agWhite)
			alphaWhiteOn.Order = 2
			alphaWhiteOn.Duration = 0.1
			alphaWhiteOn.Smoothing = "IN_OUT"
			alphaWhiteOn.Change = -1
			
			-- Animation for RuneColorGlow
			local agColor = AnimationGroup("Energize", glowColor)
			
			local alphaColorOn = Alpha("AlphaColorOn", agColor)
			alphaColorOn.Order = 1
			alphaColorOn.Duration = 0.1
			alphaColorOn.StartDelay = 0.3
			alphaColorOn.EndDelay = 4
			alphaColorOn.Smoothing = "IN_OUT"
			alphaColorOn.Change = 1
			
			local alphaColorOff = Alpha("AlphaColorOff", agColor)
			alphaColorOff.Order = 2
			alphaColorOff.Duration = 0.1
			alphaColorOff.Smoothing = "IN_OUT"
			alphaColorOff.Change = -1
			
			btn.OnSizeChanged = btn.OnSizeChanged + OnSizeChanged	

			-- ShineTimer
			local shineTimer = Timer("ShineTimer", shineTexture)
			shineTimer.Enabled = false
			shineTimer.Interval = 0.1
			shineTimer.OnTimer = shineTimer.OnTimer + OnTimer
			
			return btn
		end
	endclass "RuneButton"

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Refresh rune by index
	-- @name Refresh
	-- @class function
	------------------------------------
	function Refresh(self, runeIndex)
		if type(runeIndex) == "number" then
			if runeIndex >= 1 and runeIndex <= MAX_RUNES then
				self["Individual"..runeIndex]:Refresh()
			else
				error("Not valid rune index", 2)
			end
		else
			for i = 1, MAX_RUNES do
				self["Individual"..i]:Refresh(true)
			end
		end
	end
	
	------------------------------------
	--- Update cooldown by index
	-- @name UpdatePower
	-- @class function
	------------------------------------
	function UpdatePower(self, runeIndex, isEnergize)
		if type(runeIndex) == "number" and runeIndex >= 1 and runeIndex <= MAX_RUNES then
			self["Individual"..runeIndex]:UpdatePower(isEnergize)
		else
			error("Not valid rune index", 2)
		end
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function RunePanel(name, parent)
		local panel = Super(name, parent)
		local pct = floor(100 / MAX_RUNES) / 100
		
		panel.FrameStrata = "LOW"
		panel.Toplevel = true
		panel.Width = 130
		panel.Height = 18
		
		local btnRune, pos
		
		for i = 1, MAX_RUNES do
			btnRune = RuneButton("Individual"..i, panel)
			btnRune.ID = i
			
			panel:AddWidget(btnRune)
			
			pos = RuneBtnMapping[i]
			
			if pos < MAX_RUNES then
				panel:SetWidgetLeftWidth(btnRune, (pos-1)*pct, "pct", pct-2, "pct")
			else
				panel:SetWidgetLeftWidth(btnRune, 100 - (MAX_RUNES - 1) * pct, "pct", pct-2, "pct")
			end
		end
		
		return panel
	end
endclass "RunePanel"