﻿-- Author      : Kurapica
-- Create Date : 2012/07/22
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- ShardPanel
-- <br><br>inherit <a href="..\Base\Frame.html">Frame</a> For all methods, properties and scriptTypes
-- @name ShardPanel
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.ShardPanel", version) then
	return
end

class "ShardPanel"
	inherit "Frame"
	extend "IFShard"	
	
	SHARD_BAR_NUM_SHARDS = SHARD_BAR_NUM_SHARDS
	
	class "Shard"
		inherit "Frame"
		
		SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS		
		
		------------------------------------------------------
		-- Script
		------------------------------------------------------

		------------------------------------------------------
		-- Method
		------------------------------------------------------		
		------------------------------------
		--- Refresh the shard
		-- @name Refresh
		-- @class function
		------------------------------------
		function Refresh(self, unit)
			self.Actived = UnitPower(unit, SPELL_POWER_SOUL_SHARDS) >= self.ID
		end
		
		------------------------------------------------------
		-- Property
		------------------------------------------------------
		property "Actived" {
			Get = function(self)
				return self.__Actived or false
			end,
			Set = function(self, flag)
				if self.__Actived ~= flag then
					self.__Actived = flag
					
					if flag then
						self.SmokeA.AnimationOut.Playing = false
						self.SmokeB.AnimationOut.Playing = false
						self.Fill.AnimationOut.Playing = false
						
						self.Glow.AnimationIn.Playing = true
						self.Fill.AnimationIn.Playing = true
					else
						self.Glow.AnimationIn.Playing = false
						self.Fill.AnimationIn.Playing = false
						
						self.SmokeA.AnimationOut.Playing = true
						self.SmokeB.AnimationOut.Playing = true
						self.Fill.AnimationOut.Playing = true
					end
				end
			end,
			Type = Boolean,
		}
		
		------------------------------------------------------
		-- Script Handler
		------------------------------------------------------
		local function shardFillIn_OnPlay(self)
			self.Parent.Alpha = 0
			self.Parent.Visible = true
		end
		
		local function shardFillIn_OnFinished(self)
			self.Parent.Alpha = 1
		end
		
		local function shardFillOut_OnFinished(self)
			self.Parent.Visible = false
		end
		
		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
		function Shard(name, parent)			
			local shard = Super(name, parent)
			
			shard.Width = 52
			shard.Height = 29
			
			-- Glow
			local shardGlow = Texture("Glow", shard, "OVERLAY")
			shardGlow.TexturePath = [[Interface\PlayerFrame\UI-WarlockShard]]
			shardGlow.BlendMode = "ADD"
			shardGlow:SetTexCoord(0.01562500, 0.42187500, 0.14843750, 0.32812500)
			shardGlow.Width = 26
			shardGlow.Height = 23
			shardGlow:SetPoint("TOPLEFT", -2, 1)
			shardGlow.Alpha = 0
			
			-- SmokeA
			local shardSmokeA = Texture("SmokeA", shard, "OVERLAY")
			shardSmokeA.TexturePath = [[Interface\PlayerFrame\UI-WarlockShard]]
			shardSmokeA:SetTexCoord(0.01562500, 0.51562500, 0.34375000, 0.59375000)
			shardSmokeA.Width = 32
			shardSmokeA.Height = 32
			shardSmokeA:SetPoint("TOPLEFT", -8, 5)
			shardSmokeA.Alpha = 0
			
			-- SmokeB
			local shardSmokeB = Texture("SmokeB", shard, "OVERLAY")
			shardSmokeB.TexturePath = [[Interface\PlayerFrame\UI-WarlockShard]]
			shardSmokeB:SetTexCoord(0.01562500, 0.51562500, 0.34375000, 0.59375000)
			shardSmokeB.Width = 32
			shardSmokeB.Height = 32
			shardSmokeB:SetPoint("TOPLEFT", shardSmokeA)
			shardSmokeB.Alpha = 0
			
			-- Fill
			local shardFill = Texture("Fill", shard, "ARTWORK")
			shardFill.TexturePath = [[Interface\PlayerFrame\UI-WarlockShard]]
			shardFill.BlendMode = "ADD"
			shardFill:SetTexCoord(0.01562500, 0.28125000, 0.00781250, 0.13281250)
			shardFill.Width = 17
			shardFill.Height = 16
			shardFill:SetPoint("TOPLEFT", 3, -2)
			shardFill.Visible = false
			
			-- Border			
			local shardBorder = Texture("Border", shard, "BORDER")
			shardBorder.TexturePath = [[Interface\PlayerFrame\UI-WarlockShard]]
			shardBorder.BlendMode = "ADD"
			shardBorder:SetTexCoord(0.01562500, 0.82812500, 0.60937500, 0.83593750)
			shardBorder.Width = 52
			shardBorder.Height = 29
			shardBorder:SetPoint("TOPLEFT", -5, 3)
			
			-- Animation for shardFill
			local animInFill = AnimationGroup("AnimationIn", shardFill)
			animInFill.OnPlay = shardFillIn_OnPlay
			animInFill.OnFinished = shardFillIn_OnFinished
			
			local alphaInFill = Alpha("AlphaIn", animInFill)
			alphaInFill.Order = 1
			alphaInFill.Duration = 0.2
			alphaInFill.Change = 1
			
			local animOutFill = AnimationGroup("AnimationOut", shardFill)
			animOutFill.OnFinished = shardFillOut_OnFinished
			
			local alphaOutFill = Alpha("AlphaOut", animOutFill)
			alphaOutFill.Order = 1
			alphaOutFill.Duration = 0.2
			alphaOutFill.Change = -1
			
			-- Animation for shardGlow
			local animInGlow = AnimationGroup("AnimationIn", shardGlow)
			
			local alphaInGlow1 = Alpha("AlphaIn1", animInGlow)
			alphaInGlow1.Order = 1
			alphaInGlow1.Duration = 0.2
			alphaInGlow1.Change = 1
			
			local alphaInGlow2 = Alpha("AlphaIn2", animInGlow)
			alphaInGlow2.Order = 2
			alphaInGlow2.Duration = 0.1
			alphaInGlow2.Change = -1
			
			-- Animation for shardSmokeA
			local animOutSmokeA = AnimationGroup("AnimationOut", shardSmokeA)
			
			local alphaOutSmokeA1 = Alpha("AlphaOut1", animOutSmokeA)
			alphaOutSmokeA1.Order = 1
			alphaOutSmokeA1.Duration = 0.1
			alphaOutSmokeA1.Change = 1
			
			local alphaOutSmokeA2 = Alpha("AlphaOut2", animOutSmokeA)
			alphaOutSmokeA2.Order = 2
			alphaOutSmokeA2.Duration = 0.25
			alphaOutSmokeA2.Change = -1
			
			local roOutSmokeA = Rotation("RotationOut", animOutSmokeA)
			roOutSmokeA.Order = 1
			roOutSmokeA.Duration = 0.3
			roOutSmokeA.Radians = 1.1
			
			local scaleOutSmokeA1 = Scale("ScaleOut1", animOutSmokeA)
			scaleOutSmokeA1.Order = 1
			scaleOutSmokeA1.Duration = 0.3
			scaleOutSmokeA1.Scale = Dimension(1.2, 1.2)
			
			local scaleOutSmokeA2 = Scale("ScaleOut2", animOutSmokeA)
			scaleOutSmokeA2.Order = 2
			scaleOutSmokeA2.Duration = 0.25
			scaleOutSmokeA2.Scale = Dimension(0.4, 0.4)
			
			-- Animation for shardSmokeB
			local animOutSmokeB = AnimationGroup("AnimationOut", shardSmokeB)
			
			local alphaOutSmokeB1 = Alpha("AlphaOut1", animOutSmokeB)
			alphaOutSmokeB1.Order = 1
			alphaOutSmokeB1.Duration = 0.1
			alphaOutSmokeB1.Change = 1
			
			local alphaOutSmokeB2 = Alpha("AlphaOut2", animOutSmokeB)
			alphaOutSmokeB2.Order = 2
			alphaOutSmokeB2.Duration = 0.25
			alphaOutSmokeB2.Change = -1
			
			local roOutSmokeB = Rotation("RotationOut", animOutSmokeB)
			roOutSmokeB.Order = 1
			roOutSmokeB.Duration = 0.4
			roOutSmokeB.Radians = -0.9
			
			local scaleOutSmokeB1 = Scale("ScaleOut1", animOutSmokeB)
			scaleOutSmokeB1.Order = 1
			scaleOutSmokeB1.Duration = 0.4
			scaleOutSmokeB1.Scale = Dimension(2.5, 2.5)
			
			local scaleOutSmokeB2 = Scale("ScaleOut2", animOutSmokeB)
			scaleOutSmokeB2.Order = 2
			scaleOutSmokeB2.Duration = 0.25
			scaleOutSmokeB2.Scale = Dimension(0.4, 0.4)
			
			-- Property
			shard.__Actived = false
			
			return shard
		end
	endclass "Shard"

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Refresh totems
	-- @name Refresh
	-- @class function
	------------------------------------
	function Refresh(self)
		for i = 1, SHARD_BAR_NUM_SHARDS do
			self["Shard"..i]:Refresh(self.Unit)
		end
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function ShardPanel(name, parent)
		local panel = Super(name, parent)
		
		panel.FrameStrata = "LOW"
		panel.Toplevel = true
		panel.Width = 110
		panel.Height = 25
		
		local btnShard
		
		for i = 1, SHARD_BAR_NUM_SHARDS do
			btnShard = Shard("Shard"..i, panel)
			btnShard.ID = i
			
			if i == 1 then
				btnShard:SetPoint("TOPLEFT")
			else
				btnShard:SetPoint("TOPLEFT", self["Shard"..(i - 1)], "TOPLEFT", 35, 0)
			end
		end
		
		return panel
	end
endclass "ShardPanel"