﻿-- Author      : Kurapica
-- Create Date : 2012/07/22
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- TotemPanel
-- <br><br>inherit <a href="..\Common\LayoutPanel.html">LayoutPanel</a> For all methods, properties and scriptTypes
-- @name TotemPanel
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.TotemPanel", version) then
	return
end

class "TotemPanel"
	inherit "LayoutPanel"
	extend "IFTotem"
	
	local floor = math.floor
	
	MAX_TOTEMS = 4	
	
	class "TotemButton"
		inherit "Button"
		extend "IFCooldownIndicator"
		
		local DestroyTotem = DestroyTotem
		
		------------------------------------------------------
		-- Script
		------------------------------------------------------

		------------------------------------------------------
		-- Method
		------------------------------------------------------		
		------------------------------------
		--- Refresh the rune
		-- @name Refresh
		-- @class function
		------------------------------------
		function Refresh(self)
			local haveTotem, name, startTime, duration, icon = GetTotemInfo(slot)
			
			if haveTotem and duration > 0 then
				self.Icon.TexturePath = icon
				self.Visible = true
				
				self:OnCooldownUpdate(startTime, duration)
			else
				self.Visible = false
				
				self:OnCooldownUpdate()
			end
		end
		
		------------------------------------------------------
		-- Property
		------------------------------------------------------

		------------------------------------------------------
		-- Script Handler
		------------------------------------------------------
		local function OnClick(self, button)
			if button == "RightButton" then
				DestroyTotem(self.ID)
			end
		end
		
		local function OnEnter(self)
			if self.Visible then				
				IGAS.GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT')
				IGAS.GameTooltip:SetTotem(self.ID)
			end
		end
		
		local function OnLeave(self)
			IGAS.GameTooltip.Visible = false
		end
		
		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
		function TotemButton(name, parent)
			local btn = Super(name, parent)			
			
			btn:RegisterForClicks("AnyUp")
		
			local icon = Texture("Icon", btn, "BACKGROUND")
			icon:SetAllPoints(btn)
			
			btn.OnClick = btn.OnClick + OnClick
			btn.OnEnter = btn.OnEnter + OnEnter
			btn.OnLeave = btn.OnLeave + OnLeave
			
			return btn
		end
	endclass "TotemButton"

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Refresh totems
	-- @name Refresh
	-- @class function
	------------------------------------
	function Refresh(self)
		for i = 1, MAX_TOTEMS do
			self["Individual"..i]:Refresh()
		end
	end
	
	------------------------------------------------------
	-- Property
	------------------------------------------------------

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function TotemPanel(name, parent)
		local panel = Super(name, parent)
		local pct = floor(100 / MAX_TOTEMS) / 100
		
		panel.FrameStrata = "LOW"
		panel.Toplevel = true
		panel.Width = 130
		panel.Height = 18
		
		local btnTotem
		
		for i = 1, MAX_TOTEMS do
			btnTotem = TotemButton("Individual"..i, panel)
			btnTotem.ID = i
			
			panel:AddWidget(btnTotem)			
			
			if i < MAX_TOTEMS then
				panel:SetWidgetLeftWidth(btnTotem, (i-1)*pct, "pct", pct-2, "pct")
			else
				panel:SetWidgetLeftWidth(btnTotem, 100 - (MAX_TOTEMS - 1) * pct, "pct", pct-2, "pct")
			end
		end
		
		return panel
	end
endclass "TotemPanel"