-- Author      : Kurapica
-- Create Date : 2012/08/05
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- IFEclipse
-- @type Interface
-- @name IFEclipse
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.IFEclipse", version) then
	return
end

pcall = pcall
errorhandler = errorhandler
GetPrimaryTalentTree = GetPrimaryTalentTree or GetSpecialization
GetShapeshiftFormID = GetShapeshiftFormID
UnitAura = UnitAura

ECLIPSE_BAR_SOLAR_BUFF_ID = ECLIPSE_BAR_SOLAR_BUFF_ID
ECLIPSE_BAR_LUNAR_BUFF_ID = ECLIPSE_BAR_LUNAR_BUFF_ID
SPELL_POWER_ECLIPSE = SPELL_POWER_ECLIPSE
MOONKIN_FORM = MOONKIN_FORM

_All = "all"
_IFEclipseUnitList = _IFEclipseUnitList or UnitList(_Name)

function _IFEclipseUnitList:OnUnitListChanged()
	self:RegisterEvent("ECLIPSE_DIRECTION_CHANGE")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("UNIT_POWER")
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	
	self.OnUnitListChanged = nil
end

function _IFEclipseUnitList:ParseEvent(event, unit, powerType)
	if event == "PLAYER_TALENT_UPDATE" or event == "UPDATE_SHAPESHIFT_FORM" then
		local form = GetShapeshiftFormID()
		local needShow = false
		
		if not form then
			if GetPrimaryTalentTree() == 1 then
				needShow = true
			end
		elseif form == MOONKIN_FORM then
			needShow = true
		end
		
		self:EachK(_All, "Visible", needShow)
	elseif event == "ECLIPSE_DIRECTION_CHANGE" then
		-- unit -> isLunar
		self:EachK(_All, "Direction", unit)
	elseif event == "UNIT_AURA" and unit == "player" then
		local hasLunarEclipse = false
		local hasSolarEclipse = false
		
		local i = 1
		local name, _, _, _, _, _, _, _, _, _, spellID = UnitAura(self.Unit, i, "HELPFUL")
		
		while name do
			if spellID == ECLIPSE_BAR_SOLAR_BUFF_ID then
				hasSolarEclipse = true
			elseif spellID == ECLIPSE_BAR_LUNAR_BUFF_ID then
				hasLunarEclipse = true
			end
		
			i = i + 1
			name, _, _, _, _, _, _, _, _, _, spellID = UnitAura(self.Unit, i, "HELPFUL")
		end
	
		self:EachK(_All, "SunActived", hasSolarEclipse)
		self:EachK(_All, "MoonActived", hasLunarEclipse)
	elseif event == "UNIT_POWER" and unit == "player" and powerType == "ECLIPSE" then	
		self:EachK(_All, "Refresh")
	end	
end

interface "IFEclipse"
	extend "IFUnitElement"
	
	_IFEclipseUnitList = _IFEclipseUnitList
	
	------------------------------------------------------
	-- Enum
	------------------------------------------------------
	enum "EclipseDirection" {
		"none",
		"sun",
		"moon",
	}
	
	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	function Dispose(self)
		_IFEclipseUnitList[self] = nil
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- SunActived
	property "SunActived" {
		Get = function(self)
			return self.__SunActived or false
		end,
		Set = function(self, flag)
			self.__SunActived = flag
		end,
		Type = Boolean,
	}
	-- MoonActived
	property "MoonActived" {
		Get = function(self)
			return self.__MoonActived or false
		end,
		Set = function(self, flag)
			self.__MoonActived = flag
		end,
		Type = Boolean,
	}
	-- EclipseDirection
	property "Direction" {
		Get = function(self)
			return self.__IFEclipse_Direction or EclipseDirection.None 
		end,
		Set = function(self, dir)
			self.__IFEclipse_Direction = dir
		end,
		Type = EclipseDirection,
	}
	
	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnUnitChanged(self)
		if self.Unit and self.Unit:lower() == "player" then
			_IFEclipseUnitList[self] = _All
			self.Visible = true
		else
			_IFEclipseUnitList[self] = nil
			self.Visible = false
		end
	end
	
	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
	function IFEclipse(self)
		if select(2, UnitClass("player")) == "DRUID" then
			self.OnUnitChanged = self.OnUnitChanged + OnUnitChanged
		else
			self.Visible = false
		end
	end
endinterface "IFEclipse"