-- Author      : Kurapica
-- Create Date : 2012/06/24
-- Change Log  :

----------------------------------------------------------------------------------------------------------------------------------------
--- UnitFrame
-- <br><br>inherit <a href="..\Secure\SecureButton.html">SecureButton</a> For all methods, properties and scriptTypes
-- @name UnitFrame
----------------------------------------------------------------------------------------------------------------------------------------

-- Check Version
local version = 1
if not IGAS:NewAddon("IGAS.Widget.Unit.UnitFrame", version) then
	return
end

class "UnitFrame"
	inherit "SecureButton"
	extend "IFContainer"

	local function UpdateUnit(self, unit)
		self:Each("Unit", unit)
	end

	------------------------------------------------------
	-- Script Handler
	------------------------------------------------------
	local function OnUpdate(self, elapsed)
		self.__OnUpdateTimer = (self.__OnUpdateTimer or 0) + elapsed

		if self.__OnUpdateTimer > self.Interval then
			self.__OnUpdateTimer = 0
			return self:UpdateElements()
		end
	end

	------------------------------------------------------
	-- Script
	------------------------------------------------------

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	------------------------------------
	--- Add Element
	-- @name AddElement
	-- @class function
	-- @param ...
	------------------------------------
	function AddElement(self, ...)
		IFContainer.AddWidget(self, ...)

		UpdateUnit(self, self.Unit)
	end

	------------------------------------
	--- Insert Element
	-- @name InsertElement
	-- @class function
	-- @param ...
	------------------------------------
	function InsertElement(self, ...)
		IFContainer.InsertWidget(self, ...)

		UpdateUnit(self, self.Unit)
	end

	------------------------------------
	--- Get Element
	-- @name GetElement
	-- @class function
	-- @param ...
	------------------------------------
	function GetElement(self, ...)
		return IFContainer.GetWidget(self, ...)
	end

	------------------------------------
	--- Remove Element
	-- @name RemoveElement
	-- @class function
	-- @param ...
	------------------------------------
	function RemoveElement(self, ...)
		return IFContainer.RemoveWidget(self, ...)
	end

	------------------------------------
	--- Set Unit
	-- @name SetUnit
	-- @class function
	-- @param unit
	------------------------------------
	function SetUnit(self, unit)
		if type(unit) == "string" then
			unit = unit:lower()
		end

		if (unit == nil or type(unit) == "string") and unit ~= self:GetAttribute("unit") then
			self:SetAttribute("unit", unit)

			UpdateUnit(self, unit)

			if unit == "player" then
				self:UnregisterUnitWatch()
				self.Visible = true
			else
				self:RegisterUnitWatch()
			end

			if unit == "target" then
				self:RegisterEvent("PLAYER_TARGET_CHANGED")
				self.PLAYER_TARGET_CHANGED = UpdateElements
			else
				self:UnregisterEvent("PLAYER_TARGET_CHANGED")
				self.PLAYER_TARGET_CHANGED = nil
			end

			if unit == "mouseover" then
				self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
				self.UPDATE_MOUSEOVER_UNIT = UpdateElements
			else
				self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
				self.UPDATE_MOUSEOVER_UNIT = nil
			end

			if unit == "focus" then
				self:RegisterEvent("PLAYER_FOCUS_CHANGED")
				self.PLAYER_FOCUS_CHANGED = UpdateElements
			else
				self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
				self.PLAYER_FOCUS_CHANGED = nil
			end

			if unit:match("%w+target") or unit:match("(boss)%d?$") then
				self.OnUpdate = OnUpdate
			else
				self.OnUpdate = nil
			end
		end
	end

	------------------------------------
	--- Get Unit
	-- @name GetUnit
	-- @class function
	-- @return unit
	------------------------------------
	function GetUnit(self, unit)
		return self:GetAttribute("unit")
	end

	------------------------------------
	--- UpdateElements
	-- @name GetUnit
	-- @class function
	-- @return unit
	------------------------------------
	function UpdateElements(self)
		self:Each("Refresh")
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	-- Unit
	property "Unit" {
		Get = function(self)
			return self:GetUnit()
		end,
		Set = function(self, unit)
			self:SetUnit(unit)
		end,
		Type = String + nil,
	}
	-- Interval
	property "Interval" {
		Get = function(self)
			return self.__Interval or 0.5
		end,
		Set = function(self, int)
			if int > 0.1 then
				self.__Interval = int
			else
				self.__Interval = nil
			end
		end,
		Type = Number,
	}

	------------------------------------------------------
	-- Constructor
	------------------------------------------------------
    function UnitFrame(name, parent)
		local frame = Super(name, parent, "SecureUnitButtonTemplate")

		frame:ConvertClass(UnitFrame)

		frame.Layout = DockLayoutPanel

		frame:SetAttribute("*type1", "target")

		return frame
    end
endclass "UnitFrame"
